@@ -3,20 +3,19 @@
 from __future__ import division # confidence high
 
 try:
-    import pkg_resources
-    __version__ = pkg_resources.get_distribution('pyfits').version
+    from pyfits.version import __version__
 except:
     __version__ = ''
 
 # Import the pyfits core module.
 import pyfits.core
-import pyfits.util
 from pyfits.core import *
 from pyfits.util import *
 
 __doc__ = pyfits.core.__doc__
 
-__all__ = pyfits.core.__all__ + pyfits.util.__all__
+__all__ = pyfits.core.__all__
+
 
 try:
     import stsci.tools.tester
