@@ -1,15 +1,14 @@
-import operator
 import re
 import sys
-import warnings
 import weakref
 
 import numpy as np
 from numpy import char as chararray
 
 from pyfits.card import Card
-from pyfits.util import (lazyproperty, pairwise, _is_int, _convert_array,
-                         encode_ascii, deprecated)
+from pyfits.util import lazyproperty, pairwise, _is_int, _convert_array, \
+                        encode_ascii, deprecated
+from pyfits.verify import VerifyError
 
 
 __all__ = ['Column', 'ColDefs', 'Delayed']
@@ -35,6 +34,10 @@ NUMPY2FITS = dict([(val, key) for key, val in FITS2NUMPY.iteritems()])
 # boolean array, that should be supported
 NUMPY2FITS['b1'] = 'L'
 
+# This is the order in which values are converted to FITS types
+# Note that only double precision floating point/complex are supported
+FORMATORDER = ['L', 'B', 'I', 'J', 'K', 'D', 'M', 'A']
+
 # lists of column/field definition common names and keyword names, make
 # sure to preserve the one-to-one correspondence when updating the list(s).
 # Use lists, instead of dictionaries so the names can be displayed in a
@@ -73,16 +76,110 @@ class Delayed(object):
         return self.hdu.data[key][self.field]
 
 
+class _ColumnFormat(str):
+    """
+    Represents a FITS column format.
+
+    This is an enhancement over using a normal string for the format, since the
+    repeat count, format code, and option are available as separate attributes,
+    and smart comparison is used.  For example 1J == J.
+    """
+
+    def __new__(cls, format):
+        self = super(_ColumnFormat, cls).__new__(cls, format)
+        self.repeat, self.format, self.option = _parse_tformat(format)
+        self.format = self.format.upper()
+        return self
+
+    def __eq__(self, other):
+        if not other:
+            return False
+
+        if isinstance(other, str):
+            if not isinstance(other, _ColumnFormat):
+                try:
+                    other = _ColumnFormat(other)
+                except ValueError:
+                    return False
+        else:
+            return False
+
+        a = (self.repeat, self.format, self.option)
+        b = (other.repeat, other.format, other.option)
+        return a == b
+
+    def __hash__(self):
+        return hash(self.canonical)
+
+    @classmethod
+    def from_recformat(cls, recformat):
+        """Creates a column format from a Numpy record dtype format."""
+
+        return cls(_convert_format(recformat, reverse=True))
+
+    @lazyproperty
+    def recformat(self):
+        """Returns the equivalent Numpy record format string."""
+
+        return _convert_format(self)
+
+    @lazyproperty
+    def canonical(self):
+        """
+        Returns a 'canonical' string representation of this format.
+
+        This is in the proper form of rTa where T is the single character data
+        type code, a is the optional part, and r is the repeat.  If repeat == 1
+        (the default) it is left out of this representation.
+        """
+
+        if self.repeat == 1:
+            repeat = ''
+        else:
+            repeat = str(self.repeat)
+
+        return '%s%s%s' % (repeat, self.format, self.option)
+
+
 class _FormatX(str):
     """For X format in binary tables."""
 
     pass
 
 
+# TODO: Table column formats need to be verified upon first reading the file;
+# as it is, an invalid P format will raise a VerifyError from some deep,
+# unexpected place
 class _FormatP(str):
     """For P format in variable length table."""
 
-    pass
+    _formatp_re = re.compile(r'(?P<repeat>\d+)?P(?P<dtype>[A-Z])'
+                              '(?:\((?P<max>\d*)\))?')
+
+    def __new__(cls, dtype, repeat=None, max=None):
+        obj = super(_FormatP, cls).__new__(cls, '2i4')
+        obj.dtype = dtype
+        obj.repeat = repeat
+        obj.max = max
+        return obj
+
+    @classmethod
+    def from_tform(cls, format):
+        m = cls._formatp_re.match(format)
+        if not m or m.group('dtype') not in FITS2NUMPY:
+            raise VerifyError('Invalid column format: %s' % format)
+        repeat = m.group('repeat')
+        array_dtype = m.group('dtype')
+        max = m.group('max')
+        if not max:
+            max = None
+        return cls(FITS2NUMPY[array_dtype], repeat=repeat, max=max)
+
+    @property
+    def tform(self):
+        repeat = '' if self.repeat is None else self.repeat
+        max = '' if self.max is None else self.max
+        return '%sP%s(%s)' % (repeat, NUMPY2FITS[self.dtype], max)
 
 
 class Column(object):
@@ -92,8 +189,8 @@ class Column(object):
     Does not support `theap` yet.
     """
 
-    def __init__(self, name=None, format=None, unit=None, null=None, \
-                       bscale=None, bzero=None, disp=None, start=None, \
+    def __init__(self, name=None, format=None, unit=None, null=None,
+                       bscale=None, bzero=None, disp=None, start=None,
                        dim=None, array=None):
         """
         Construct a `Column` by specifying attributes.  All attributes
@@ -130,9 +227,6 @@ class Column(object):
             column dimension corresponding to ``TDIM`` keyword
         """
 
-        if format is None:
-            raise ValueError('Must specify format to construct Column.')
-
         # any of the input argument (except array) can be a Card or just
         # a number/string
         for attr in KEYWORD_ATTRIBUTES:
@@ -145,78 +239,65 @@ class Column(object):
 
         # if the column data is not ndarray, make it to be one, i.e.
         # input arrays can be just list or tuple, not required to be ndarray
-        try:
-            # legit FITS format? convert to record format (e.g. '3J'->'3i4')
-            recfmt = _convert_format(format)
-        except ValueError:
-            try:
-                # legit recarray format?
-                recfmt = format
-                format = _convert_format(recfmt, reverse=True)
-            except ValueError:
-                raise ValueError('Illegal format `%s`.' % format)
+        if format is not None:
+            # check format
+            if not isinstance(format, _ColumnFormat):
+                try:
 
-        self.format = format
-        # Zero-length formats are legal in the FITS format, but since they
-        # are not supported by numpy we mark columns that use them as
-        # "phantom" columns, that are not considered when reading the data
-        # as a record array.
-        if self.format[0] == '0' or \
-           (self.format[-1] == '0' and self.format[-2].isalpha()):
-            self._phantom = True
-        else:
-            self._phantom = False
-
-        if isinstance(dim, basestring):
-            self._dims = _parse_tdim(dim)
-        elif isinstance(dim, tuple):
-            self._dims = dim
-        elif not dim:
-            self._dims = tuple()
-        else:
-            raise TypeError(
-                "`dim` argument must be a string containing a valid value "
-                "for the TDIMn header keyword associated with this column, "
-                "or a tuple containing the C-order dimensions for the column")
-
-        if self._dims:
-            repeat = _parse_tformat(format)[0]
-            if reduce(operator.mul, self._dims) > repeat:
-                raise ValueError(
-                    "The repeat count of the column format %r for column %r "
-                    "is fewer than the number of elements per the TDIM "
-                    "argument %r." % (name, format, dim))
-        # does not include Object array because there is no guarantee
-        # the elements in the object array are consistent.
-        if not isinstance(array,
-                          (np.ndarray, chararray.chararray, Delayed)):
-            try:  # try to convert to a ndarray first
-                if array is not None:
-                    array = np.array(array)
-            except:
-                try:  # then try to convert it to a strings array
-                    itemsize = int(recformat[1:])
-                    array = chararray.array(array, itemsize=itemsize)
+                    # legit FITS format?
+                    format = _ColumnFormat(format)
+                    recformat = format.recformat
                 except ValueError:
-                    # then try variable length array
-                    if isinstance(recformat, _FormatP):
-                        array = _VLF(array, dtype=recformat.dtype)
-                    else:
-                        raise ValueError('Data is inconsistent with the '
-                                         'format `%s`.' % format)
+                    try:
+                        # legit recarray format?
+                        recformat = format
+                        format = _ColumnFormat.from_recformat(format)
+                    except ValueError:
+                        raise ValueError('Illegal format `%s`.' % format)
 
+            self.format = format
+            # Zero-length formats are legal in the FITS format, but since they
+            # are not supported by numpy we mark columns that use them as
+            # "phantom" columns, that are not considered when reading the data
+            # as a record array.
+            if self.format[0] == '0' or \
+               (self.format[-1] == '0' and self.format[-2].isalpha()):
+                self._phantom = True
+            else:
+                self._phantom = False
+
+            # does not include Object array because there is no guarantee
+            # the elements in the object array are consistent.
+            if not isinstance(array,
+                              (np.ndarray, chararray.chararray, Delayed)):
+                try:  # try to convert to a ndarray first
+                    if array is not None:
+                        array = np.array(array)
+                except:
+                    try:  # then try to conver it to a strings array
+                        itemsize = int(recformat[1:])
+                        array = chararray.array(array, itemsize=itemsize)
+                    except ValueError:
+                        # then try variable length array
+                        if isinstance(recformat, _FormatP):
+                            array = _VLF(array, dtype=recformat.dtype)
+                        else:
+                            raise ValueError('Data is inconsistent with the '
+                                             'format `%s`.' % format)
+
+        else:
+            raise ValueError('Must specify format to construct Column.')
 
         # scale the array back to storage values if there is bscale/bzero
         if isinstance(array, np.ndarray):
             # make a copy if scaled, so as not to corrupt the original array
             if bzero not in ['', None, 0] or bscale not in ['', None, 1]:
-                array = array.copy()
                 if bzero not in ['', None, 0]:
-                    array += -bzero
+                    array = array - bzero
                 if bscale not in ['', None, 1]:
-                    array /= bscale
+                    array = array / bscale
 
-        array = self._convert_to_valid_data_type(array)
+        array = self._convert_to_valid_data_type(array, self.format)
         self.array = array
 
     def __repr__(self):
@@ -227,32 +308,43 @@ class Column(object):
                 text += attr + ' = ' + repr(value) + '; '
         return text[:-2]
 
+    def __eq__(self, other):
+        """
+        Two columns are equal if their name and format are the same.  Other
+        attributes aren't taken into account at this time.
+        """
+
+        # According to the FITS standard column names must be case-insensitive
+        a = (self.name.lower(), self.format)
+        b = (other.name.lower(), other.format)
+        return a == b
+
+    def __hash__(self):
+        """
+        Like __eq__, the hash of a column should be based on the unique column
+        name and format, and be case-insensitive with respect to the column name.
+        """
+
+        return hash((self.name.lower(), self.format))
+
     def copy(self):
         """
         Return a copy of this `Column`.
         """
-        tmp = Column(format='I') # just use a throw-away format
+        tmp = Column(format='I')  # just use a throw-away format
         tmp.__dict__ = self.__dict__.copy()
         return tmp
 
-    def _convert_to_valid_data_type(self, array):
+    def _convert_to_valid_data_type(self, array, format):
         # Convert the format to a type we understand
         if isinstance(array, Delayed):
             return array
         elif array is None:
             return array
         else:
-            format = self.format
-            dims = self._dims
             if 'A' in format and 'P' not in format:
                 if array.dtype.char in 'SU':
-                    if dims:
-                        # The 'last' dimension (first in the order given
-                        # in the TDIMn keyword itself) is the number of
-                        # characters in each string
-                        fsize = dims[-1]
-                    else:
-                        fsize = int(_convert_format(format)[1:])
+                    fsize = int(_convert_format(format)[1:])
                     return chararray.array(array, itemsize=fsize)
                 else:
                     numpy_format = _convert_format(format)
@@ -309,7 +401,7 @@ class ColDefs(object):
         input :
             An existing table HDU, an existing ColDefs, or recarray
 
-        **(Deprecated)** tbtype : str, optional
+        **(Deprecated)** tbtype : str (optional)
             which table HDU, ``"BinTableHDU"`` (default) or
             ``"TableHDU"`` (text table).
             Now ColDefs for a normal (binary) table by default, but converted
@@ -372,8 +464,8 @@ class ColDefs(object):
             # go through header keywords to pick out column definition keywords
             # definition dictionaries for each field
             col_attributes = [{} for i in range(nfields)]
-            for card in hdr.ascard:
-                key = TDEF_RE.match(card.key)
+            for keyword, value in hdr.iteritems():
+                key = TDEF_RE.match(keyword)
                 try:
                     keyword = key.group('label')
                 except:
@@ -383,7 +475,7 @@ class ColDefs(object):
                     if col <= nfields and col > 0:
                         idx = KEYWORD_NAMES.index(keyword)
                         attr = KEYWORD_ATTRIBUTES[idx]
-                        col_attributes[col - 1][attr] = card.value
+                        col_attributes[col - 1][attr] = value
 
             # data reading will be delayed
             for col in range(nfields):
@@ -416,9 +508,9 @@ class ColDefs(object):
         raise AttributeError(name)
 
     @property
-    @deprecated(message='The %(func)s attribute is deprecated; use the '
-                        '%(alternative)s attribute instead.',
-                alternative='.columns')
+    @deprecated('3.0', message='The %(func)s attribute is deprecated; use the '
+                               '%(alternative)s attribute instead.',
+                alternative='`ColDefs.columns`')
     def data(self):
         """
         What was originally self.columns is now self.data; this provides some
@@ -435,12 +527,6 @@ class ColDefs(object):
     def _recformats(self):
         return [_convert_format(fmt) for fmt in self.formats]
 
-    @lazyproperty
-    def _dims(self):
-        """Returns the values of the TDIMn keywords parsed into tuples."""
-
-        return [col._dims for col in self.columns]
-
     def __getitem__(self, key):
         x = self.columns[key]
         if _is_int(key):
@@ -510,7 +596,7 @@ class ColDefs(object):
         assert isinstance(column, Column)
 
         for cname in KEYWORD_ATTRIBUTES:
-            attr = getattr(self, cname+'s')
+            attr = getattr(self, cname + 's')
             attr.append(getattr(column, cname))
 
         self._arrays.append(column.array)
@@ -535,7 +621,7 @@ class ColDefs(object):
         indx = _get_index(self.names, col_name)
 
         for cname in KEYWORD_ATTRIBUTES:
-            attr = getattr(self, cname+'s')
+            attr = getattr(self, cname + 's')
             del attr[indx]
 
         del self._arrays[indx]
@@ -564,7 +650,7 @@ class ColDefs(object):
         """
 
         indx = _get_index(self.names, col_name)
-        getattr(self, attrib+'s')[indx] = new_value
+        getattr(self, attrib + 's')[indx] = new_value
 
         # If this ColDefs is being tracked by a Table, inform the
         # table that its data is now invalid.
@@ -640,7 +726,7 @@ class ColDefs(object):
             for idx in range(len(lst)):
                 lst[idx] = lst[idx].strip().lower()
                 if lst[idx][-1] == 's':
-                    lst[idx]=list[idx][:-1]
+                    lst[idx] = list[idx][:-1]
 
         ret = {}
 
@@ -662,8 +748,8 @@ class ColDefs(object):
 class _ASCIIColDefs(ColDefs):
     """ColDefs implementation for ASCII tables."""
 
-    _ascii_fmt = {'A':'A1', 'I':'I10', 'J':'I15', 'E':'E15.7', 'F':'F16.7',
-                  'D':'D25.17'}
+    _ascii_fmt = {'A': 'A1', 'I': 'I10', 'J': 'I15', 'E': 'E15.7',
+                  'F': 'F16.7', 'D': 'D25.17'}
 
     _padding_byte = ' '
 
@@ -720,24 +806,34 @@ class _ASCIIColDefs(ColDefs):
 class _VLF(np.ndarray):
     """Variable length field object."""
 
-    def __new__(cls, args):
+    def __new__(cls, input, dtype='a'):
         """
         Parameters
         ----------
-        args
+        input
             a sequence of variable-sized elements.
         """
 
-        a = np.array(args, dtype=np.object)
-        self = np.ndarray.__new__(cls, shape=(len(args)), buffer=a,
+        if dtype == 'a':
+            try:
+                # this handles ['abc'] and [['a','b','c']]
+                # equally, beautiful!
+                input = map(lambda x: chararray.array(x, itemsize=1), input)
+            except:
+                raise ValueError('Inconsistent input data array: %s' % input)
+
+        a = np.array(input, dtype=np.object)
+        self = np.ndarray.__new__(cls, shape=(len(input)), buffer=a,
                                   dtype=np.object)
-        self._max = 0
+        self.max = 0
+        self.element_dtype = dtype
         return self
 
     def __array_finalize__(self, obj):
         if obj is None:
             return
-        self._max = obj._max
+        self.max = obj.max
+        self.element_dtype = obj.element_dtype
 
     def __setitem__(self, key, value):
         """
@@ -749,12 +845,12 @@ class _VLF(np.ndarray):
             pass
         elif isinstance(value, chararray.chararray) and value.itemsize == 1:
             pass
-        elif self._dtype == 'a':
+        elif self.element_dtype == 'a':
             value = chararray.array(value, itemsize=1)
         else:
-            value = np.array(value, dtype=self._dtype)
+            value = np.array(value, dtype=self.element_dtype)
         np.ndarray.__setitem__(self, key, value)
-        self._max = max(self._max, len(value))
+        self.max = max(self.max, len(value))
 
 
 def _get_index(names, key):
@@ -790,7 +886,7 @@ def _get_index(names, key):
             # try to match case-insentively,
             _key = key.lower().rstrip()
             names = [n.lower().rstrip() for n in names]
-            count = names.count(_key) # occurrence of _key in names
+            count = names.count(_key)  # occurrence of _key in names
             if count == 1:
                 indx = names.index(_key)
             elif count == 0:
@@ -820,12 +916,12 @@ def _unwrapx(input, output, nx):
     """
 
     pow2 = np.array([128, 64, 32, 16, 8, 4, 2, 1], dtype='uint8')
-    nbytes = ((nx-1) // 8) + 1
+    nbytes = ((nx - 1) // 8) + 1
     for i in range(nbytes):
-        _min = i*8
-        _max = min((i+1)*8, nx)
+        _min = i * 8
+        _max = min((i + 1) * 8, nx)
         for j in range(_min, _max):
-            output[...,j] = np.bitwise_and(input[...,i], pow2[j-i*8])
+            output[..., j] = np.bitwise_and(input[..., i], pow2[j - i * 8])
 
 
 def _wrapx(input, output, nx):
@@ -844,22 +940,22 @@ def _wrapx(input, output, nx):
         number of bits
     """
 
-    output[...] = 0 # reset the output
-    nbytes = ((nx-1) // 8) + 1
-    unused = nbytes*8 - nx
+    output[...] = 0  # reset the output
+    nbytes = ((nx - 1) // 8) + 1
+    unused = nbytes * 8 - nx
     for i in range(nbytes):
-        _min = i*8
-        _max = min((i+1)*8, nx)
+        _min = i * 8
+        _max = min((i + 1) * 8, nx)
         for j in range(_min, _max):
             if j != _min:
-                np.left_shift(output[...,i], 1, output[...,i])
-            np.add(output[...,i], input[...,j], output[...,i])
+                np.left_shift(output[..., i], 1, output[..., i])
+            np.add(output[..., i], input[..., j], output[..., i])
 
     # shift the unused bits
-    np.left_shift(output[...,i], unused, output[...,i])
+    np.left_shift(output[..., i], unused, output[..., i])
 
 
-def _makep(input, desp_output, dtype, nrows=None):
+def _makep(input, desp_output, format, nrows=None):
     """
     Construct the P format column array, both the data descriptors and
     the data.  It returns the output "data" array of data type `dtype`.
@@ -877,8 +973,8 @@ def _makep(input, desp_output, dtype, nrows=None):
         output "descriptor" array of data type ``Int32``--must be nrows wide in
         its first dimension
 
-    dtype
-        data type of the variable array
+    format
+        the _FormatP object reperesenting the format of the variable array
 
     nrows : int, optional
         number of rows to create in the column; defaults to the number of rows
@@ -891,30 +987,29 @@ def _makep(input, desp_output, dtype, nrows=None):
         nrows = len(input)
     n = min(len(input), nrows)
 
-    data_output = _VLF([None] * nrows)
-    data_output._dtype = dtype
+    data_output = _VLF([None] * nrows, dtype=format.dtype)
 
-    if dtype == 'a':
+    if format.dtype == 'a':
         _nbytes = 1
     else:
-        _nbytes = np.array([], dtype=dtype).itemsize
+        _nbytes = np.array([], dtype=np.typeDict[format.dtype]).itemsize
 
     for idx in range(nrows):
         if idx < len(input):
             rowval = input[idx]
         else:
-            if dtype == 'a':
-                rowval = ' ' * data_output._max
+            if format.dtype == 'a':
+                rowval = ' ' * data_output.max
             else:
-                rowval = [0] * data_output._max
-        if dtype == 'a':
+                rowval = [0] * data_output.max
+        if format.dtype == 'a':
             data_output[idx] = chararray.array(encode_ascii(rowval),
                                                itemsize=1)
         else:
-            data_output[idx] = np.array(rowval, dtype=dtype)
+            data_output[idx] = np.array(rowval, dtype=format.dtype)
 
-        desp_output[idx,0] = len(data_output[idx])
-        desp_output[idx,1] = _offset
+        desp_output[idx, 0] = len(data_output[idx])
+        desp_output[idx, 1] = _offset
         _offset += len(data_output[idx]) * _nbytes
 
     return data_output
@@ -926,8 +1021,10 @@ def _parse_tformat(tform):
     try:
         (repeat, dtype, option) = TFORMAT_RE.match(tform.strip()).groups()
     except:
-        warnings.warn('Format "%s" is not recognized.' % tform)
-
+        # TODO: Maybe catch this error use a default type (bytes, maybe?) for
+        # unrecognized column types.  As long as we can determine the correct
+        # byte width somehow..
+        raise VerifyError('Format "%s" is not recognized.' % tform)
 
     if repeat == '':
         repeat = 1
@@ -937,18 +1034,53 @@ def _parse_tformat(tform):
     return (repeat, dtype, option)
 
 
-def _parse_tdim(tdim):
-    """Parse the ``TDIM`` value into a tuple (may return an empty tuple if
-    the value ``TDIM`` value is empty or invalid).
+def _scalar_to_format(value):
+    """
+    Given a scalar value or string, returns the minimum FITS column format
+    that can represent that value.  'minimum' is defined by the order given in
+    FORMATORDER.
     """
 
-    m = tdim and TDIM_RE.match(tdim)
-    if m:
-        dims = m.group('dims')
-        return tuple(int(d.strip()) for d in dims.split(','))[::-1]
+    # TODO: Numpy 1.6 and up has a min_scalar_type() function that can handle
+    # this; in the meantime we have to use our own implementation (which for
+    # now is pretty naive)
 
-    # Ignore any dim values that don't specify a multidimensional column
-    return tuple()
+    # First, if value is a string, try to convert to the appropriate scalar
+    # value
+    for type_ in (int, float, complex):
+        try:
+            value = type_(value)
+            break
+        except ValueError:
+            continue
+
+    if isinstance(value, int) and value in (0, 1):
+        # Could be a boolean
+        return 'L'
+    elif isinstance(value, int):
+        for char in ('B', 'I', 'J', 'K'):
+            type_ = np.dtype(FITS2NUMPY[char]).type
+            if type_(value) == value:
+                return char
+    elif isinstance(value, float):
+        # For now just assume double precision
+        return 'D'
+    elif isinstance(value, complex):
+        return 'M'
+    else:
+        return 'A' + str(len(value))
+
+
+def _cmp_recformats(f1, f2):
+    """
+    Compares two numpy recformats using the ordering given by FORMATORDER.
+    """
+
+    if f1[0] == 'a' and f2[0] == 'a':
+        return cmp(int(f1[1:]), int(f2[1:]))
+    else:
+        f1, f2 = NUMPY2FITS[f1], NUMPY2FITS[f2]
+        return cmp(FORMATORDER.index(f1), FORMATORDER.index(f2))
 
 
 def _convert_fits2record(format):
@@ -968,20 +1100,19 @@ def _convert_fits2record(format):
                  # make sure option is integer
                 output_format = FITS2NUMPY[dtype] + str(int(option))
         else:
-            _repeat = ''
-            if _repeat != 1:
-                _repeat = str(repeat)
-            output_format = _repeat + FITS2NUMPY[dtype]
+            repeat_str = ''
+            if repeat != 1:
+                repeat_str = str(repeat)
+            output_format = repeat_str + FITS2NUMPY[dtype]
 
     elif dtype == 'X':
-        nbytes = ((repeat-1) // 8) + 1
+        nbytes = ((repeat - 1) // 8) + 1
         # use an array, even if it is only ONE u1 (i.e. use tuple always)
         output_format = _FormatX(repr((nbytes,)) + 'u1')
         output_format._nx = repeat
 
     elif dtype == 'P':
-        output_format = _FormatP('2i4')
-        output_format._dtype = FITS2NUMPY[option[0]]
+        output_format = _FormatP.from_tform(format)
     elif dtype == 'F':
         output_format = 'f8'
     else:
@@ -1021,9 +1152,7 @@ def _convert_record2fits(format):
         ntot = int(repeat) * int(option)
 
         output_format = str(ntot) + NUMPY2FITS[dtype]
-    elif isinstance(dtype, _FormatX):
-        warnings.warn('X format')
-    elif dtype + option in NUMPY2FITS: # record format
+    elif dtype + option in NUMPY2FITS:  # record format
         if repeat != 1:
             repeat = str(repeat)
         else:
@@ -1034,6 +1163,7 @@ def _convert_record2fits(format):
 
     return output_format
 
+
 def _convert_format(format, reverse=False):
     """
     Convert FITS format spec to record format spec.  Do the opposite if
