@@ -2,7 +2,6 @@ from __future__ import division
 from __future__ import with_statement
 
 import gzip
-import mmap
 import os
 import sys
 import tempfile
@@ -13,10 +12,10 @@ import zipfile
 import numpy as np
 from numpy import memmap as Memmap
 
-from pyfits.util import (Extendable, isreadable, iswritable, isfile,
-                         fileobj_open, fileobj_name, fileobj_closed,
-                         fileobj_mode, _array_from_file, _array_to_file,
-                         _write_string, deprecated, encode_ascii)
+from pyfits.util import (isreadable, iswritable, isfile, fileobj_open,
+                         fileobj_name, fileobj_closed, fileobj_mode,
+                         _array_from_file, _array_to_file, _write_string,
+                         deprecated)
 
 
 # File object open modes
@@ -47,11 +46,6 @@ class _File(object):
     Represents a FITS file on disk (or in some other file-like object).
     """
 
-    __metaclass__ = Extendable
-
-    # See self._test_mmap
-    _mmap_available = None
-
     def __init__(self, fileobj=None, mode='readonly', memmap=False):
         if fileobj is None:
             self.__file = None
@@ -77,12 +71,7 @@ class _File(object):
                 # name does not begin with a drive letter (Windows), try to
                 # get it over the web.
                 #
-            try:
-                self.name, _ = urllib.urlretrieve(fileobj)
-            except (TypeError, ValueError):
-                # A couple different exceptions can occur here when passing a
-                # filename into urlretrieve in Python 3
-                raise IOError('File does not exist: %r' % fileobj)
+            self.name, _ = urllib.urlretrieve(fileobj)
         else:
             self.name = fileobj_name(fileobj)
 
@@ -100,11 +89,84 @@ class _File(object):
 
         # Initialize the internal self.__file object
         if isfile(fileobj) or isinstance(fileobj, gzip.GzipFile):
-            self._open_fileobj(fileobj, mode)
+            closed = fileobj_closed(fileobj)
+            fmode = fileobj_mode(fileobj) or PYTHON_MODES[mode]
+
+            if not closed:
+                # In some cases (like on Python 3) a file opened for
+                # appending still shows a mode of 'r+', hence the extra
+                # check for the append case
+                if ((mode == 'append' and fmode not in ('ab+', 'rb+')) or
+                    (mode != 'append' and PYTHON_MODES[mode] != fmode)):
+                    raise ValueError(
+                        "Input mode '%s' (%s) does not match mode of the "
+                        "input file (%s)." % (mode, PYTHON_MODES[mode], fmode))
+                self.__file = fileobj
+            elif isfile(fileobj):
+                self.__file = fileobj_open(self.name, PYTHON_MODES[mode])
+                # Return to the beginning of the file--in Python 3 when
+                # opening in append mode the file pointer is at the end of
+                # the file
+                self.__file.seek(0)
+            else:
+                self.__file = gzip.open(self.name, PYTHON_MODES[mode])
         elif isinstance(fileobj, basestring):
-            self._open_filename(fileobj, mode)
+            if os.path.exists(self.name):
+                with fileobj_open(self.name, 'rb') as f:
+                    magic = f.read(4)
+            else:
+                magic = ''.encode('raw-unicode-escape')
+            ext = os.path.splitext(self.name)[1]
+            if ext == '.gz' or magic.startswith(GZIP_MAGIC):
+                # Handle gzip files
+                if mode in ['update', 'append']:
+                    raise IOError(
+                          "Writing to gzipped fits files is not currently "
+                          "supported")
+                self.__file = gzip.open(self.name)
+                self.compression = 'gzip'
+            elif ext == '.zip' or magic.startswith(PKZIP_MAGIC):
+                # Handle zip files
+                if mode in ['update', 'append']:
+                    raise IOError(
+                          "Writing to zipped fits files is not currently "
+                          "supported")
+                zfile = zipfile.ZipFile(self.name)
+                namelist = zfile.namelist()
+                if len(namelist) != 1:
+                    raise IOError(
+                      "Zip files with multiple members are not supported.")
+                self.__file = tempfile.NamedTemporaryFile(suffix='.fits')
+                self.__file.write(zfile.read(namelist[0]))
+                zfile.close()
+                self.compression = 'zip'
+            else:
+                self.__file = fileobj_open(self.name, PYTHON_MODES[mode])
+                # Make certain we're back at the beginning of the file
+            self.__file.seek(0)
         else:
-            self._open_filelike(fileobj, mode)
+            # We are dealing with a file like object.
+            # Assume it is open.
+            self.file_like = True
+            self.__file = fileobj
+
+            # If there is not seek or tell methods then set the mode to
+            # output streaming.
+            if (not hasattr(self.__file, 'seek') or
+                not hasattr(self.__file, 'tell')):
+                self.mode = mode = 'ostream'
+
+            if (self.mode in ('copyonwrite', 'update', 'append') and
+                not hasattr(self.__file, 'write')):
+                raise IOError("File-like object does not have a 'write' "
+                              "method, required for mode '%s'."
+                              % self.mode)
+
+            if (self.mode in ('readonly', 'denywrite') and
+                not hasattr(self.__file, 'read')):
+                raise IOError("File-like object does not have a 'read' "
+                              "method, required for mode %r."
+                              % self.mode)
 
         if isinstance(fileobj, gzip.GzipFile):
             self.compression = 'gzip'
@@ -132,15 +194,8 @@ class _File(object):
             self.size = self.__file.tell()
             self.__file.seek(pos)
 
-        if self.memmap:
-            if not isfile(self.__file):
-                self.memmap = False
-                warnings.warn('Disabling mmap for non-file-backed file-like '
-                              'object.')
-            elif not self.readonly and not self._test_mmap():
-                # Test mmap.flush--see
-                # https://github.com/astropy/astropy/issues/968
-                self.memmap = False
+        if self.memmap and not isfile(self.__file):
+            self.memmap = False
 
     def __repr__(self):
         return '<%s.%s %s>' % (self.__module__, self.__class__.__name__,
@@ -153,9 +208,10 @@ class _File(object):
     def __exit__(self, type, value, traceback):
         self.close()
 
-    @deprecated
+    @deprecated('3.0', message='This method should not be treated as public.')
     def getfile(self):
-        """**Deprecated** Will be going away as soon as I figure out how."""
+        """Will be going away as soon as I figure out how."""
+
         return self.__file
 
     def readable(self):
@@ -284,126 +340,3 @@ class _File(object):
             self.__file.close()
 
         self.closed = True
-
-    def _open_fileobj(self, fileobj, mode):
-        """Open a FITS file from a file object or a GzipFile object."""
-
-        closed = fileobj_closed(fileobj)
-        fmode = fileobj_mode(fileobj) or PYTHON_MODES[mode]
-
-        if not closed:
-            # In some cases (like on Python 3) a file opened for appending
-            # still shows a mode of 'r+', hence the extra check for the append
-            # case
-            if ((mode == 'append' and fmode not in ('ab+', 'rb+')) or
-                (mode != 'append' and PYTHON_MODES[mode] != fmode)):
-                raise ValueError(
-                    "Input mode '%s' (%s) does not match mode of the "
-                    "input file (%s)." % (mode, PYTHON_MODES[mode], fmode))
-            self.__file = fileobj
-        elif isfile(fileobj):
-            self.__file = fileobj_open(self.name, PYTHON_MODES[mode])
-            # Return to the beginning of the file--in Python 3 when opening in
-            # append mode the file pointer is at the end of the file
-            self.__file.seek(0)
-        else:
-            self.__file = gzip.open(self.name, PYTHON_MODES[mode])
-
-    def _open_filelike(self, fileobj, mode):
-        """Open a FITS file from a file-like object, i.e. one that has
-        read and/or write methods.
-        """
-
-        self.file_like = True
-        self.__file = fileobj
-
-        # If there is not seek or tell methods then set the mode to
-        # output streaming.
-        if (not hasattr(self.__file, 'seek') or
-            not hasattr(self.__file, 'tell')):
-            self.mode = mode = 'ostream'
-
-        if (self.mode in ('copyonwrite', 'update', 'append') and
-            not hasattr(self.__file, 'write')):
-            raise IOError("File-like object does not have a 'write' "
-                          "method, required for mode '%s'."
-                          % self.mode)
-
-        if (self.mode in ('readonly', 'denywrite') and
-            not hasattr(self.__file, 'read')):
-            raise IOError("File-like object does not have a 'read' "
-                          "method, required for mode %r."
-                          % self.mode)
-
-    def _open_filename(self, filename, mode):
-        """Open a FITS file from a filename string."""
-
-        if os.path.exists(self.name):
-            with fileobj_open(self.name, 'rb') as f:
-                magic = f.read(4)
-        else:
-            magic = ''.encode('raw-unicode-escape')
-        ext = os.path.splitext(self.name)[1]
-        if ext == '.gz' or magic.startswith(GZIP_MAGIC):
-            # Handle gzip files
-            self.__file = gzip.open(self.name, PYTHON_MODES[mode])
-            self.compression = 'gzip'
-        elif ext == '.zip' or magic.startswith(PKZIP_MAGIC):
-            # Handle zip files
-            if mode in ['update', 'append']:
-                raise IOError(
-                      "Writing to zipped fits files is not currently "
-                      "supported")
-            zfile = zipfile.ZipFile(self.name)
-            namelist = zfile.namelist()
-            if len(namelist) != 1:
-                raise IOError(
-                  "Zip files with multiple members are not supported.")
-            self.__file = tempfile.NamedTemporaryFile(suffix='.fits')
-            self.__file.write(zfile.read(namelist[0]))
-            zfile.close()
-            self.compression = 'zip'
-        else:
-            self.__file = fileobj_open(self.name, PYTHON_MODES[mode])
-            # Make certain we're back at the beginning of the file
-        self.__file.seek(0)
-
-    def _test_mmap(self):
-        """Tests that mmap, and specifically mmap.flush works.  This may
-        be the case on some uncommon platforms (see
-        https://github.com/astropy/astropy/issues/968).
-
-        If mmap.flush is found not to work, ``self.memmap = False`` is
-        set and a warning is issued.
-        """
-
-        if self._mmap_available is not None:
-            return self._mmap_available
-
-        tmpfd, tmpname = tempfile.mkstemp()
-        try:
-            # Windows does not allow mappings on empty files
-            os.write(tmpfd, encode_ascii(' '))
-            os.fsync(tmpfd)
-            try:
-                mm = mmap.mmap(tmpfd, 1, access=mmap.ACCESS_WRITE)
-            except mmap.error, e:
-                warnings.warn('Failed to create mmap: %s; mmap use will be '
-                              'disabled' % str(e))
-                _File._mmap_available = False
-                return False
-            try:
-                mm.flush()
-            except mmap.error:
-                warnings.warn('mmap.flush is unavailable on this platform; '
-                              'using mmap in writeable mode will be disabled')
-                _File._mmap_available = False
-                return False
-            finally:
-                mm.close()
-        finally:
-            os.close(tmpfd)
-            os.remove(tmpname)
-
-        _File._mmap_available = True
-        return True
