@@ -5,9 +5,9 @@ import weakref
 
 import numpy as np
 
-from pyfits.column import (ASCIITNULL, FITS2NUMPY, Column, ColDefs, _FormatX,
-                           _FormatP, _VLF, _get_index, _wrapx, _unwrapx,
-                            _convert_ascii_format)
+from pyfits.column import ASCIITNULL, FITS2NUMPY, TDIM_RE, Column, ColDefs, \
+                          _FormatX, _FormatP, _VLF, _get_index, _wrapx, \
+                          _unwrapx, _convert_format, _convert_ascii_format
 from pyfits.util import _array_from_file, decode_ascii, lazyproperty
 
 
@@ -112,7 +112,6 @@ class FITS_record(object):
             outlist.append(repr(self[idx]))
         return '(%s)' % ', '.join(outlist)
 
-
     def field(self, field):
         """
         Get the field data of the record.
@@ -120,7 +119,6 @@ class FITS_record(object):
 
         return self.__getitem__(field)
 
-
     def setfield(self, field, value):
         """
         Set the field data of the record.
@@ -176,8 +174,8 @@ class FITS_rec(np.recarray):
         self._nfields = len(self.dtype.names)
         self._convert = [None] * len(self.dtype.names)
         self._heapoffset = 0
-        self._heapsize = 0
         self._file = None
+        self._buffer = None
         self._coldefs = None
         self._gap = 0
         self.names = list(self.dtype.names)
@@ -191,8 +189,8 @@ class FITS_rec(np.recarray):
         if isinstance(obj, FITS_rec):
             self._convert = obj._convert
             self._heapoffset = obj._heapoffset
-            self._heapsize = obj._heapsize
             self._file = obj._file
+            self._buffer = obj._buffer
             self._coldefs = obj._coldefs
             self._nfields = obj._nfields
             self._gap = obj._gap
@@ -205,8 +203,8 @@ class FITS_rec(np.recarray):
             self._convert = [None] * len(obj.dtype.names)
 
             self._heapoffset = getattr(obj, '_heapoffset', 0)
-            self._heapsize = getattr(obj, '_heapsize', 0)
             self._file = getattr(obj, '_file', None)
+            self._buffer = getattr(obj, '_buffer', None)
 
             self._coldefs = None
             self._gap = 0
@@ -304,11 +302,11 @@ class FITS_rec(np.recarray):
                 self.field(self.names[idx])[row] = value.field(self.names[idx])
         elif isinstance(value, (tuple, list)):
             if self._nfields == len(value):
-                for idx in range (self._nfields):
+                for idx in range(self._nfields):
                     self.field(idx)[row] = value[idx]
             else:
-               raise ValueError('Input tuple or list required to have %s '
-                                'elements.' % self._nfields)
+                raise ValueError('Input tuple or list required to have %s '
+                                 'elements.' % self._nfields)
         else:
             raise TypeError('Assignment requires a FITS_record, tuple, or '
                             'list as input.')
@@ -347,7 +345,7 @@ class FITS_rec(np.recarray):
         # recursion
         field = np.recarray.field(base, indx)
 
-        if (self._convert[indx] is None):
+        if self._convert[indx] is None:
             # for X format
             if isinstance(recformat, _FormatX):
                 _nx = recformat._nx
@@ -361,36 +359,40 @@ class FITS_rec(np.recarray):
 
             # for P format
             if isinstance(recformat, _FormatP):
-                dummy = _VLF([None] * len(self))
-                dummy._dtype = recformat._dtype
-                raw_data = self._get_raw_data()
-                if raw_data is None:
-                    raise IOError(
-                        "Could not find heap data for the %r variable-length "
-                        "array column." % self.names[indx])
+                dummy = _VLF([None] * len(self), dtype=recformat.dtype)
                 for i in range(len(self)):
-                    offset = field[i, 1] + self._heapoffset
-                    count = field[i, 0]
+                    _offset = field[i, 1] + self._heapoffset
 
-                    if recformat._dtype == 'a':
-                        dt = np.dtype(recformat._dtype + str(1))
-                        arr_len = count * dt.itemsize
-                        da = raw_data[offset:offset + arr_len].view(dt)
-                        da = np.char.array(da.view(dtype=dt), itemsize=count)
-                        dummy[i] = decode_ascii(da)
+                    if self._file is not None:
+                        self._file.seek(_offset)
+                        def get_pdata(dtype, count):
+                            return _array_from_file(self._file, dtype=dtype,
+                                                    count=count, sep='')
+                    else:  # There must be a _buffer or something is wrong
+                        buff = self._buffer[_offset:]
+                        def get_pdata(dtype, count):
+                            return np.fromstring(buff, dtype=dtype,
+                                                 count=count, sep='')
+
+                    if recformat.dtype == 'a':
+                        count = field[i, 0]
+                        dt = recformat.dtype + str(1)
+                        da = get_pdata(dt, count)
+                        dummy[i] = np.char.array(da, itemsize=count)
+                        dummy[i] = decode_ascii(dummy[i])
                     else:
-                        dt = np.dtype(recformat._dtype)
-                        arr_len = count * dt.itemsize
-                        dummy[i] = raw_data[offset:offset + arr_len].view(dt)
+                        count = field[i, 0]
+                        dt = recformat.dtype
+                        dummy[i] = get_pdata(dt, count)
                         dummy[i].dtype = dummy[i].dtype.newbyteorder('>')
 
                 # scale by TSCAL and TZERO
                 if _scale or _zero:
                     for i in range(len(self)):
-                        dummy[i][:] = dummy[i]*bscale+bzero
+                        dummy[i][:] = dummy[i] * bscale + bzero
 
                 # Boolean (logical) column
-                if recformat._dtype is FITS2NUMPY['L']:
+                if recformat.dtype == FITS2NUMPY['L']:
                     for i in range(len(self)):
                         dummy[i] = np.equal(dummy[i], ord('T'))
 
@@ -409,13 +411,7 @@ class FITS_rec(np.recarray):
                                       'E'.encode('ascii'))
                 dummy = np.where(dummy.strip() == nullval, str(ASCIITNULL),
                                  dummy)
-                try:
-                    dummy = np.array(dummy, dtype=_type)
-                except ValueError, e:
-                    raise ValueError(
-                        '%s; the header may be missing the necessary TNULL%d '
-                        'keyword or the table contains invalid data' %
-                        (e, indx + 1))
+                dummy = np.array(dummy, dtype=_type)
 
                 self._convert[indx] = dummy
             else:
@@ -442,9 +438,9 @@ class FITS_rec(np.recarray):
                         actual_nitems = dummy.itemsize
                     else:
                         actual_nitems = dummy.shape[1]
-                    if nitems > actual_nitems:
+                    if nitems != actual_nitems:
                         warnings.warn(
-                            'TDIM%d value %s does not fit with the size of '
+                        'TDIM%d value %s does not fit with the size of '
                             'the array items (%d).  TDIM%d will be ignored.'
                             % (indx + 1, self._coldefs.dims[indx],
                                actual_nitems, indx + 1))
@@ -460,7 +456,7 @@ class FITS_rec(np.recarray):
                                 self._convert[indx])
                 if _zero:
                     self._convert[indx] += bzero
-            elif _bool and dummy.dtype != bool:
+            elif _bool:
                 self._convert[indx] = np.equal(dummy, ord('T'))
             elif _str:
                 try:
@@ -469,9 +465,8 @@ class FITS_rec(np.recarray):
                     pass
 
             if dim:
-                nitems = reduce(operator.mul, dim)
                 if self._convert[indx] is None:
-                    self._convert[indx] = dummy[:,:nitems]
+                    self._convert[indx] = dummy
                 if _str:
                     fmt = self._convert[indx].dtype.char
                     dtype = ('|%s%d' % (fmt, dim[-1]), dim[:-1])
@@ -494,30 +489,6 @@ class FITS_rec(np.recarray):
         hdu = new_table(self._coldefs, nrows=shape[0])
         return hdu.data
 
-    def _get_raw_data(self):
-        """
-        Returns the base array of self that "raw data array" that is the
-        array in the format that it was first read from a file before it was
-        sliced or viewed as a different type in any way.
-
-        This is determined by walking through the bases until finding one that
-        has at least the same number of bytes as self, plus the heapsize.  This
-        may be the immediate .base but is not always.  This is used primarily
-        for variable-length array support which needs to be able to find the
-        heap (the raw data *may* be larger than nbytes + heapsize if it
-        contains a gap or padding).
-
-        May return ``None`` if no array resembling the "raw data" according to
-        the stated criteria can be found.
-        """
-
-        raw_data_bytes = self.nbytes + self._heapsize
-        base = self
-        while hasattr(base, 'base') and base.base is not None:
-            base = base.base
-            if hasattr(base, 'nbytes') and base.nbytes >= raw_data_bytes:
-                return base
-
     def _get_scale_factors(self, indx):
         """
         Get the scaling flags and factors for one field.
@@ -541,8 +512,15 @@ class FITS_rec(np.recarray):
             bscale = 1
         if not _zero:
             bzero = 0
-
-        dim = self._coldefs._dims[indx]
+        dim = self._coldefs.dims[indx]
+        m = dim and TDIM_RE.match(dim)
+        if m:
+            dim = m.group('dims')
+            dim = tuple(int(d.strip()) for d in dim.split(','))[::-1]
+        else:
+            # Ignore any dim values that don't specify a multidimensional
+            # column
+            dim = ''
 
         return (_str, _bool, _number, _scale, _zero, bscale, bzero, dim)
 
@@ -564,6 +542,7 @@ class FITS_rec(np.recarray):
                 widths.append(f[1])
             loc.append(loc[-1] + super(FITS_rec, self).field(idx).itemsize)
 
+        self._heapsize = 0
         for indx in range(len(self.dtype.names)):
             recformat = self._coldefs._recformats[indx]
             field = super(FITS_rec, self).field(indx)
@@ -581,16 +560,10 @@ class FITS_rec(np.recarray):
             # add the location offset of the heap area for each
             # variable length column
             if isinstance(recformat, _FormatP):
-                # Reset the heapsize and recompute it starting from the first P
-                # column
-                if indx == 0:
-                    self._heapsize = 0
-
                 field[:] = 0  # reset
                 npts = map(len, self._convert[indx])
-
-                dtype = np.array([], dtype=recformat._dtype).dtype
                 field[:len(npts), 0] = npts
+                dtype = np.array([], dtype=recformat.dtype)
                 field[1:, 1] = (np.add.accumulate(field[:-1, 0]) *
                                 dtype.itemsize)
                 field[:, 1][:] += self._heapsize
@@ -619,8 +592,8 @@ class FITS_rec(np.recarray):
                         raise ValueError(
                             'Column `%s` starting point overlaps to the '
                             'previous column.' % indx + 1)
-                    trail = loc[indx+1] - widths[indx] - \
-                             self._coldefs.starts[indx]
+                    trail = (loc[indx + 1] - widths[indx] -
+                             self._coldefs.starts[indx])
                     if trail < 0:
                         raise ValueError(
                             'Column `%s` ending point overlaps to the next '
@@ -637,7 +610,7 @@ class FITS_rec(np.recarray):
                     # result is not allowed to expand (as C/Python does).
                     for jdx in range(len(dummy)):
                         x = fmt % dummy[jdx]
-                        if len(x) > (loc[indx+1] - loc[indx]):
+                        if len(x) > (loc[indx + 1] - loc[indx]):
                             raise ValueError(
                                 "Number `%s` does not fit into the output's "
                                 "itemsize of %s." % (x, widths[indx]))
@@ -663,20 +636,9 @@ class FITS_rec(np.recarray):
                         for idx in xrange(len(dummy)):
                             val = dummy[idx]
                             dummy[idx] = val + (pad * (itemsize - len(val)))
-
-                        # Encode *after* handling the padding byte or else
-                        # Numpy will complain about trying to append bytes to
-                        # an array
-                        if dummy.dtype.kind == 'U':
-                            dummy = dummy.encode('ascii')
-
-                    if field.shape == dummy.shape:
-                        field[:] = dummy
-                    else:
-                        # Reshaping the data is necessary in cases where the
-                        # TDIMn keyword was used to shape a column's entries
-                        # into arrays
-                        field[:] = dummy.ravel().view(field.dtype)
+                    if dummy.dtype != field.dtype:
+                        dummy = dummy.astype(field.dtype)
+                    field[:] = dummy
 
                 del dummy
 
@@ -684,5 +646,4 @@ class FITS_rec(np.recarray):
             elif _bool:
                 field[:] = np.choose(self._convert[indx],
                                      (np.array([ord('F')], dtype=np.int8)[0],
-                                      np.array([ord('T')],dtype=np.int8)[0]))
-
+                                      np.array([ord('T')], dtype=np.int8)[0]))
