@@ -5,8 +5,8 @@ from pyfits.column import Column, ColDefs, FITS2NUMPY
 from pyfits.fitsrec import FITS_rec, FITS_record
 from pyfits.hdu.image import _ImageBaseHDU, PrimaryHDU
 from pyfits.hdu.table import _TableLikeHDU
-from pyfits.util import (lazyproperty, _is_int, _pad_length,
-                         _is_pseudo_unsigned)
+from pyfits.util import lazyproperty, _is_int, _pad_length, \
+                        _is_pseudo_unsigned
 
 
 class Group(FITS_record):
@@ -253,13 +253,13 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
         if len(self._axes) <= 0:
             self._axes = [0]
             self._header['NAXIS'] = 1
-            self._header.update('NAXIS1', 0, after='NAXIS')
+            self._header.set('NAXIS1', 0, after='NAXIS')
 
     @classmethod
     def match_header(cls, header):
-        card = header.ascard[0]
-        return card.key == 'SIMPLE' and 'GROUPS' in header and \
-               header['GROUPS'] == True
+        keyword = header.cards[0].keyword
+        return (keyword == 'SIMPLE' and 'GROUPS' in header and
+                header['GROUPS'] == True)
 
     @lazyproperty
     def data(self):
@@ -328,7 +328,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
         # Only really a lazyproperty for symmetry with _TableBaseHDU
         return 0
 
-    # 0.6.5.5
+    @property
     def size(self):
         """
         Returns the size (in bytes) of the HDU's data part.
@@ -369,13 +369,13 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
         # add NAXISi if it does not exist
         for idx, axis in enumerate(self._axes):
             try:
-                self._header['NAXIS'+ str(idx + 1)] = axis
+                self._header['NAXIS' + str(idx + 1)] = axis
             except KeyError:
                 if (idx == 0):
                     after = 'NAXIS'
-                else :
+                else:
                     after = 'NAXIS' + str(idx)
-                self._header.update('NAXIS' + str(idx + 1), axis, after=after)
+                self._header.set('NAXIS' + str(idx + 1), axis, after=after)
 
         # delete extra NAXISi's
         for idx in range(len(self._axes) + 1, old_naxis + 1):
@@ -384,30 +384,27 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
             except KeyError:
                 pass
 
-        if self._data_loaded and isinstance(self.data, GroupData):
-            self._header.update('GROUPS', True,
-                                after='NAXIS' + str(len(self._axes)))
-            self._header.update('PCOUNT', len(self.data.parnames),
-                                after='GROUPS')
-            self._header.update('GCOUNT', len(self.data), after='PCOUNT')
+        if isinstance(self.data, GroupData):
+            self._header.set('GROUPS', True,
+                             after='NAXIS' + str(len(self._axes)))
+            self._header.set('PCOUNT', len(self.data.parnames), after='GROUPS')
+            self._header.set('GCOUNT', len(self.data), after='PCOUNT')
             npars = len(self.data.parnames)
             scale, zero = self.data._get_scale_factors(npars)[3:5]
             if scale:
-                self._header.update('BSCALE',
-                                    self.data._coldefs.bscales[npars])
+                self._header.set('BSCALE', self.data._coldefs.bscales[npars])
             if zero:
-                self._header.update('BZERO',
-                                    self.data._coldefs.bzeros[npars])
+                self._header.set('BZERO', self.data._coldefs.bzeros[npars])
             for idx in range(npars):
-                self._header.update('PTYPE' + str(idx + 1),
-                                    self.data.parnames[idx])
+                self._header.set('PTYPE' + str(idx + 1),
+                                 self.data.parnames[idx])
                 scale, zero = self.data._get_scale_factors(idx)[3:5]
                 if scale:
-                    self._header.update('PSCAL' + str(idx + 1),
-                                        self.data._coldefs.bscales[idx])
+                    self._header.set('PSCAL' + str(idx + 1),
+                                     self.data._coldefs.bscales[idx])
                 if zero:
-                    self._header.update('PZERO' + str(idx + 1),
-                                        self.data._coldefs.bzeros[idx])
+                    self._header.set('PZERO' + str(idx + 1),
+                                     self.data._coldefs.bzeros[idx])
 
         # Update the position of the EXTEND keyword if it already exists
         if 'EXTEND' in self._header:
@@ -415,7 +412,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
                 after = 'NAXIS' + str(len(self._axes))
             else:
                 after = 'NAXIS'
-            self._header.update('EXTEND', self._header['EXTEND'], after=after)
+            self._header.set('EXTEND', after=after)
 
     def _get_tbdata(self):
         # get the right shape for the data part of the random group,
@@ -525,7 +522,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
             # yet.  We can handle that in a generic manner so we do it in the
             # base class.  The other possibility is that there is no data at
             # all.  This can also be handled in a gereric manner.
-            return super(GroupsHDU,self)._calculate_datasum(blocking=blocking)
+            return super(GroupsHDU, self)._calculate_datasum(blocking=blocking)
 
     def _summary(self):
         summary = super(GroupsHDU, self)._summary()
