@@ -1,5 +1,6 @@
 import gzip
 import os
+import shutil
 import sys
 import warnings
 
@@ -16,16 +17,13 @@ from pyfits.hdu.compressed import CompImageHDU
 from pyfits.hdu.groups import GroupsHDU
 from pyfits.hdu.image import PrimaryHDU, ImageHDU
 from pyfits.hdu.table import _TableBaseHDU
-from pyfits.util import (Extendable, _is_int, _tmp_name, _with_extensions,
-                         _pad_length, BLOCK_SIZE, isfile, fileobj_name,
-                         fileobj_closed, fileobj_mode, ignore_sigint,
-                         _get_array_mmap, indent)
-from pyfits.verify import _Verify, _ErrList, VerifyError
+from pyfits.util import (_is_int, _tmp_name, _pad_length, BLOCK_SIZE, isfile,
+                         fileobj_name, fileobj_closed, fileobj_mode,
+                         ignore_sigint, _get_array_memmap, indent)
+from pyfits.verify import _Verify, _ErrList, VerifyError, VerifyWarning
 
 
-@_with_extensions
-def fitsopen(name, mode='readonly', memmap=None, classExtensions={},
-             **kwargs):
+def fitsopen(name, mode='readonly', memmap=None, save_backup=False, **kwargs):
     """Factory function to open a FITS file and return an `HDUList` object.
 
     Parameters
@@ -45,10 +43,11 @@ def fitsopen(name, mode='readonly', memmap=None, classExtensions={},
     memmap : bool
         Is memory mapping to be used?
 
-    classExtensions : dict (''Deprecated'')
-        A dictionary that maps pyfits classes to extensions of those
-        classes.  When present in the dictionary, the extension class
-        will be constructed in place of the pyfits class.
+    save_backup : bool
+        If the file was opened in update or append mode, this ensures that a
+        backup of the original file is saved before any changes are flushed.
+        The backup has the same name as the original file with ".bak" appended.
+        If "file.bak" already exists then "file.bak.1" is used, and so on.
 
     kwargs : dict
         optional keyword arguments, possible values are:
@@ -69,14 +68,15 @@ def fitsopen(name, mode='readonly', memmap=None, classExtensions={},
             Do not issue an exception when opening a file that is
             missing an ``END`` card in the last header.
 
-        - **checksum** : bool
+        - **checksum** : bool, str
 
             If `True`, verifies that both ``DATASUM`` and
             ``CHECKSUM`` card values (when present in the HDU header)
             match the header and data of all HDU's in the file.  Updates to a
-            file that already has a checksum will NOT be preserved unless the
-            file was opened with ``checksum=True``.  This behavior may change
-            in a future PyFITS version.
+            file that already has a checksum will preserve and update the
+            existing checksums unless this argument is given a value of
+            'remove', in which case the CHECKSUM and DATASUM values are not
+            checked, and are removed when saving changes to the file.
 
         - **disable_image_compression** : bool
 
@@ -88,6 +88,14 @@ def fitsopen(name, mode='readonly', memmap=None, classExtensions={},
             If `True`, image data is not scaled using BSCALE/BZERO values
             when read.
 
+        - **scale_back** : bool
+
+            If `True`, when saving changes to a file that contained scaled
+            image data, restore the data to the original type and reapply the
+            original BSCALE/BZERO values.  This could lead to loss of accuracy
+            if scaling back to integer values after performing floating point
+            operations on the data.
+
     Returns
     -------
         hdulist : an `HDUList` object
@@ -96,6 +104,10 @@ def fitsopen(name, mode='readonly', memmap=None, classExtensions={},
 
     """
 
+    if memmap is None:
+        from pyfits import USE_MEMMAP
+        memmap = USE_MEMMAP
+
     if 'uint16' in kwargs and 'uint' not in kwargs:
         kwargs['uint'] = kwargs['uint16']
         del kwargs['uint16']
@@ -103,7 +115,7 @@ def fitsopen(name, mode='readonly', memmap=None, classExtensions={},
     if not name:
         raise ValueError('Empty filename: %s' % repr(name))
 
-    return HDUList.fromfile(name, mode, memmap, **kwargs)
+    return HDUList.fromfile(name, mode, memmap, save_backup, **kwargs)
 
 
 class HDUList(list, _Verify):
@@ -112,8 +124,6 @@ class HDUList(list, _Verify):
     file is opened, a `HDUList` object is returned.
     """
 
-    __metaclass__ = Extendable
-
     def __init__(self, hdus=[], file=None):
         """
         Construct a `HDUList` object.
@@ -129,6 +139,8 @@ class HDUList(list, _Verify):
         """
 
         self.__file = file
+        self._save_backup = False
+
         if hdus is None:
             hdus = []
 
@@ -151,8 +163,7 @@ class HDUList(list, _Verify):
         for idx in range(len(self)):
             yield self[idx]
 
-    @_with_extensions
-    def __getitem__(self, key, classExtensions={}):
+    def __getitem__(self, key):
         """
         Get an HDU from the `HDUList`, indexed by number or name.
         """
@@ -225,66 +236,48 @@ class HDUList(list, _Verify):
         self.close()
 
     @classmethod
-    def fromfile(cls, fileobj, mode="copyonwrite", memmap=False, **kwargs):
+    def fromfile(cls, fileobj, mode='readonly', memmap=False,
+                 save_backup=False, **kwargs):
         """
         Creates an HDUList instance from a file-like object.
 
-        The actual implementation of `fitsopen()`.
+        The actual implementation of :func:`fitsopen`, and generally shouldn't
+        be used directly.  Use :func:`pyfits.open` instead (and see its
+        documentation for details of the parameters accepted by this method).
         """
 
-        # instantiate a FITS file object (ffo)
-        ffo = _File(fileobj, mode=mode, memmap=memmap)
-        hdulist = cls(file=ffo)
+        return cls._readfrom(fileobj=fileobj, mode=mode, memmap=memmap,
+                             save_backup=save_backup, **kwargs)
 
-        saved_compression_enabled = compressed.COMPRESSION_ENABLED
+    @classmethod
+    def fromstring(cls, data, **kwargs):
+        """
+        Creates an HDUList instance from a string or other in-memory data
+        buffer containing an entire FITS file.  Similar to
+        :meth:`HDUList.fromfile`, but does not accept the mode or memmap
+        arguments, as they are only relevant to reading from a file on disk.
 
-        try:
-            if 'disable_image_compression' in kwargs and \
-               kwargs['disable_image_compression']:
-                compressed.COMPRESSION_ENABLED = False
+        This is useful for interfacing with other libraries such as CFITSIO,
+        and may also be useful for streaming applications.
 
-            if mode == 'ostream':
-                # Output stream--not interested in reading/parsing the HDUs--just
-                # writing to the output file
-                return hdulist
+        Parameters
+        ----------
+        data : str, buffer, memoryview, etc.
+            A string or other memory buffer containing an entire FITS file.  It
+            should be noted that if that memory is read-only (such as a Python
+            string) the returned :class:`HDUList`'s data portions will also be
+            read-only.
 
-            # read all HDUs
-            while True:
-                try:
-                    hdu = _BaseHDU.readfrom(ffo, **kwargs)
-                    hdulist.append(hdu)
-                    hdu._new = False
-                except EOFError:
-                    break
-                # check in the case there is extra space after the last HDU or
-                # corrupted HDU
-                except (VerifyError, ValueError), err:
-                    warnings.warn(
-                        'Error validating header for HDU #%d (note: PyFITS '
-                        'uses zero-based indexing).\n%s\n'
-                        'There may be extra bytes after the last HDU or the '
-                        'file is corrupted.' %
-                        (len(hdulist), indent(str(err))))
-                    break
-                except IOError, err:
-                    if ffo.writeonly:
-                        break
-                    else:
-                        raise
+        kwargs : dict
+            Optional keyword arguments.  See :func:`pyfits.open` for details.
 
-            # If we're trying to read only and no header units were found,
-            # raise and exception
-            if mode in ('readonly', 'denywrite') and len(hdulist) == 0:
-                raise IOError('Empty or corrupt FITS file')
+        Returns
+        -------
+        hdul : HDUList
+            An :class:`HDUList` object representing the in-memory FITS file.
+        """
 
-            # initialize/reset attributes to be used in "update/append" mode
-            hdulist._resize = False
-            hdulist._truncate = False
-
-        finally:
-            compressed.COMPRESSION_ENABLED = saved_compression_enabled
-
-        return hdulist
+        return cls._readfrom(data=data, **kwargs)
 
     def fileinfo(self, index):
         """
@@ -300,7 +293,7 @@ class HDUList(list, _Verify):
 
         Returns
         -------
-        dictionary or None
+        fileinfo : dict or None
 
             The dictionary details information about the locations of
             the indexed HDU within an associated file.  Returns `None`
@@ -308,20 +301,20 @@ class HDUList(list, _Verify):
 
             Dictionary contents:
 
-            ========== =========================================================
+            ========== ========================================================
             Key        Value
-            ========== =========================================================
+            ========== ========================================================
             file       File object associated with the HDU
             filename   Name of associated file object
-            filemode   Mode in which the file was opened (readonly, copyonwrite,
-                       update, append, denywrite, ostream)
+            filemode   Mode in which the file was opened (readonly,
+                       copyonwrite, update, append, denywrite, ostream)
             resized    Flag that when `True` indicates that the data has been
                        resized since the last read/write so the returned values
                        may not be valid.
             hdrLoc     Starting byte location of header in file
             datLoc     Starting byte location of data block in file
             datSpan    Data size including padding
-            ========== =========================================================
+            ========== ========================================================
 
         """
 
@@ -336,12 +329,12 @@ class HDUList(list, _Verify):
                 f = None
 
                 for hdu in self:
-                   info = hdu.fileinfo()
+                    info = hdu.fileinfo()
 
-                   if info:
-                      f = info['file']
-                      fm = info['filemode']
-                      break
+                    if info:
+                        f = info['file']
+                        fm = info['filemode']
+                        break
 
                 output = {'file': f, 'filemode': fm, 'hdrLoc': None,
                           'datLoc': None, 'datSpan': None}
@@ -353,8 +346,7 @@ class HDUList(list, _Verify):
 
         return output
 
-    @_with_extensions
-    def insert(self, index, hdu, classExtensions={}):
+    def insert(self, index, hdu):
         """
         Insert an HDU into the `HDUList` at the given `index`.
 
@@ -365,11 +357,6 @@ class HDUList(list, _Verify):
 
         hdu : _BaseHDU instance
             The HDU object to insert
-
-        classExtensions : dict
-            A dictionary that maps pyfits classes to extensions of those
-            classes.  When present in the dictionary, the extension class
-            will be constructed in place of the pyfits class.
         """
 
         if not isinstance(hdu, _BaseHDU):
@@ -382,12 +369,12 @@ class HDUList(list, _Verify):
                 # We are inserting a new Primary HDU so we need to
                 # make the current Primary HDU into an extension HDU.
                 if isinstance(self[0], GroupsHDU):
-                   raise ValueError(
-                         "The current Primary HDU is a GroupsHDU.  "
-                         "It can't be made into an extension HDU, "
-                         "so you can't insert another HDU in front of it.")
+                    raise ValueError(
+                        "The current Primary HDU is a GroupsHDU.  "
+                        "It can't be made into an extension HDU, "
+                        "so another HDU cannot be inserted before it.")
 
-                hdu1= ImageHDU(self[0].data, self[0].header)
+                hdu1 = ImageHDU(self[0].data, self[0].header)
 
                 # Insert it into position 1, then delete HDU at position 0.
                 super(HDUList, self).insert(1, hdu1)
@@ -409,8 +396,8 @@ class HDUList(list, _Verify):
                     index = 1
         else:
             if isinstance(hdu, GroupsHDU):
-               raise ValueError('A GroupsHDU must be inserted as a '
-                                'Primary HDU.')
+                raise ValueError('A GroupsHDU must be inserted as a '
+                                 'Primary HDU.')
 
             if isinstance(hdu, PrimaryHDU):
                 # You passed a Primary HDU but we need an Extension HDU
@@ -424,8 +411,7 @@ class HDUList(list, _Verify):
         # make sure the EXTEND keyword is in primary HDU if there is extension
         self.update_extend()
 
-    @_with_extensions
-    def append(self, hdu, classExtensions={}):
+    def append(self, hdu):
         """
         Append a new HDU to the `HDUList`.
 
@@ -433,11 +419,6 @@ class HDUList(list, _Verify):
         ----------
         hdu : instance of _BaseHDU
             HDU to add to the `HDUList`.
-
-        classExtensions : dict
-            A dictionary that maps pyfits classes to extensions of those
-            classes.  When present in the dictionary, the extension class
-            will be constructed in place of the pyfits class.
         """
 
         if not isinstance(hdu, _BaseHDU):
@@ -445,8 +426,8 @@ class HDUList(list, _Verify):
 
         if len(self) > 0:
             if isinstance(hdu, GroupsHDU):
-               raise ValueError(
-                     "Can't append a GroupsHDU to a non-empty HDUList")
+                raise ValueError(
+                    "Can't append a GroupsHDU to a non-empty HDUList")
 
             if isinstance(hdu, PrimaryHDU):
                 # You passed a Primary HDU but we need an Extension HDU
@@ -512,9 +493,7 @@ class HDUList(list, _Verify):
             name = hdu.name
             if isinstance(name, str):
                 name = name.strip().upper()
-            # 'PRIMARY' should always work as a reference to the first HDU
-            if ((name == _key or (_key == 'PRIMARY' and idx == 0)) and
-                (_ver is None or _ver == hdu._extver)):
+            if name == _key and (_ver is None or _ver == hdu._extver):
                 found = idx
                 nfound += 1
 
@@ -535,10 +514,8 @@ class HDUList(list, _Verify):
             if hdu.data is not None:
                 continue
 
-    @_with_extensions
     @ignore_sigint
-    def flush(self, output_verify='fix', verbose=False,
-              classExtensions={}):
+    def flush(self, output_verify='fix', verbose=False):
         """
         Force a write of the `HDUList` back to the file (for append and
         update modes only).
@@ -552,12 +529,6 @@ class HDUList(list, _Verify):
 
         verbose : bool
             When `True`, print verbose messages
-
-        classExtensions : dict
-            A dictionary that maps pyfits classes to extensions of
-            those classes.  When present in the dictionary, the
-            extension class will be constructed in place of the pyfits
-            class.
         """
 
         if self.__file.mode not in ('append', 'update', 'ostream'):
@@ -565,6 +536,24 @@ class HDUList(list, _Verify):
                           % self.__file.mode)
             return
 
+        if self._save_backup and self.__file.mode in ('append', 'update'):
+            filename = self.__file.name
+            if os.path.exists(filename):
+                # The the file doesn't actually exist anymore for some reason
+                # then there's no point in trying to make a backup
+                backup = filename + '.bak'
+                idx = 1
+                while os.path.exists(backup):
+                    backup = filename + '.bak.' + str(idx)
+                    idx += 1
+                warnings.warn('Saving a backup of %s to %s.' %
+                              (filename, backup))
+                try:
+                    shutil.copy(filename, backup)
+                except IOError, e:
+                    raise IOError('Failed to save backup to destination %s: '
+                                  '%s' % (filename, str(e)))
+
         self.verify(option=output_verify)
 
         if self.__file.mode in ('append', 'ostream'):
@@ -577,14 +566,7 @@ class HDUList(list, _Verify):
 
                 # only append HDU's which are "new"
                 if hdu._new:
-                    # only output the checksum if flagged to do so
-                    if hasattr(hdu, '_output_checksum'):
-                        checksum = hdu._output_checksum
-                    else:
-                        checksum = False
-
-                    # TODO: Fix this once new HDU writing API is settled on
-                    hdu._prewriteto(checksum=checksum)
+                    hdu._prewriteto(checksum=hdu._output_checksum)
                     try:
                         hdu._writeto(self.__file)
                         if verbose:
@@ -616,14 +598,13 @@ class HDUList(list, _Verify):
                 hdr['EXTEND'] = True
         elif len(self) > 1:
             if hdr['NAXIS'] == 0:
-                hdr.update('EXTEND', True, after='NAXIS')
+                hdr.set('EXTEND', True, after='NAXIS')
             else:
                 n = hdr['NAXIS']
-                hdr.update('EXTEND', True, after='NAXIS' + str(n))
+                hdr.set('EXTEND', True, after='NAXIS' + str(n))
 
-    @_with_extensions
     def writeto(self, fileobj, output_verify='exception', clobber=False,
-                classExtensions={}, checksum=False):
+                checksum=False):
         """
         Write the `HDUList` to a new file.
 
@@ -650,8 +631,6 @@ class HDUList(list, _Verify):
             warnings.warn("There is nothing to write.")
             return
 
-        if output_verify == 'warn':
-            output_verify = 'exception'
         self.verify(option=output_verify)
 
         # check if the file object is closed
@@ -696,7 +675,6 @@ class HDUList(list, _Verify):
                 hdu._postwriteto()
         hdulist.close(output_verify=output_verify, closed=closed)
 
-
     def close(self, output_verify='exception', verbose=False, closed=True):
         """
         Close the associated FITS file and memmap object, if any.
@@ -731,10 +709,10 @@ class HDUList(list, _Verify):
 
         Parameters
         ----------
-        output : file, bool (optional)
-            A file-like object to write the output to.  If ``False``, does not
+        output : file, optional
+            A file-like object to write the output to.  If False, does not
             output to a file and instead returns a list of tuples representing
-            the HDU info.  Writes to ``sys.stdout`` by default.
+            the HDU info.  Writes to sys.stdout by default.
         """
 
         if output is None:
@@ -779,10 +757,91 @@ class HDUList(list, _Verify):
                    None.
         """
         if self.__file is not None:
-           if hasattr(self.__file, 'name'):
-              return self.__file.name
+            if hasattr(self.__file, 'name'):
+                return self.__file.name
         return None
 
+    @classmethod
+    def _readfrom(cls, fileobj=None, data=None, mode='readonly',
+                  memmap=False, save_backup=False, **kwargs):
+        """
+        Provides the implementations from HDUList.fromfile and
+        HDUList.fromstring, both of which wrap this method, as their
+        implementations are largely the same.
+        """
+
+        if fileobj is not None:
+            # instantiate a FITS file object (ffo)
+            ffo = _File(fileobj, mode=mode, memmap=memmap)
+            hdulist = cls(file=ffo)
+        else:
+            hdulist = cls()
+            # This method is currently only called from HDUList.fromstring and
+            # HDUList.fromfile.  If fileobj is None then this must be the
+            # fromstring case; the data type of `data` will be checked in the
+            # _BaseHDU.fromstring call.
+
+        hdulist._save_backup = save_backup
+
+        saved_compression_enabled = compressed.COMPRESSION_ENABLED
+
+        try:
+            if ('disable_image_compression' in kwargs and
+                kwargs['disable_image_compression']):
+                compressed.COMPRESSION_ENABLED = False
+
+            if mode == 'ostream':
+                # Output stream--not interested in reading/parsing the
+                # HDUs--just writing to the output file
+                return hdulist
+
+            # read all HDUs
+            while True:
+                try:
+                    if fileobj is not None:
+                        try:
+                            hdu = _BaseHDU.readfrom(ffo, **kwargs)
+                        except EOFError:
+                            break
+                        except IOError, err:
+                            if ffo.writeonly:
+                                break
+                            else:
+                                raise
+                    else:
+                        if not data:
+                            break
+                        hdu = _BaseHDU.fromstring(data)
+                        data = data[hdu._datLoc + hdu._datSpan:]
+                    hdulist.append(hdu)
+                    hdu._new = False
+                    if 'checksum' in kwargs:
+                        hdu._output_checksum = kwargs['checksum']
+                # check in the case there is extra space after the last HDU or
+                # corrupted HDU
+                except (VerifyError, ValueError), err:
+                    warnings.warn(
+                        'Error validating header for HDU #%d (note: PyFITS '
+                        'uses zero-based indexing).\n%s\n'
+                        'There may be extra bytes after the last HDU or the '
+                        'file is corrupted.' %
+                        (len(hdulist), indent(str(err))), VerifyWarning)
+                    break
+
+            # If we're trying to read only and no header units were found,
+            # raise and exception
+            if mode in ('readonly', 'denywrite') and len(hdulist) == 0:
+                raise IOError('Empty or corrupt FITS file')
+
+            # initialize/reset attributes to be used in "update/append" mode
+            hdulist._resize = False
+            hdulist._truncate = False
+
+        finally:
+            compressed.COMPRESSION_ENABLED = saved_compression_enabled
+
+        return hdulist
+
     def _verify(self, option='warn'):
         text = ''
         errs = _ErrList([], unit='HDU')
@@ -812,7 +871,7 @@ class HDUList(list, _Verify):
                     after = 'NAXIS'
                 else:
                     after = 'NAXIS' + str(naxis)
-                header.update('EXTEND', value=True, after=after)
+                header.set('EXTEND', value=True, after=after)
 
             errs.append(self.run_option(option, err_text=err_text,
                                         fix_text=fix_text, fix=fix))
@@ -837,11 +896,7 @@ class HDUList(list, _Verify):
         for hdu in self:
             # Need to all _prewriteto() for each HDU first to determine if
             # resizing will be necessary
-            if hasattr(hdu, '_output_checksum'):
-                checksum = hdu._output_checksum
-            else:
-                checksum = False
-            hdu._prewriteto(checksum=checksum, inplace=True)
+            hdu._prewriteto(checksum=hdu._output_checksum, inplace=True)
 
         try:
             self._wasresized()
@@ -857,7 +912,7 @@ class HDUList(list, _Verify):
 
             # reset the modification attributes after updating
             for hdu in self:
-                hdu._header._mod = False
+                hdu._header._modified = False
         finally:
             for hdu in self:
                 hdu._postwriteto()
@@ -890,7 +945,7 @@ class HDUList(list, _Verify):
             if sys.platform.startswith('win'):
                 # Collect a list of open mmaps to the data; this well be used
                 # later.  See below.
-                mmaps = [(idx, _get_array_mmap(hdu.data), hdu.data)
+                mmaps = [(idx, _get_array_memmap(hdu.data), hdu.data)
                          for idx, hdu in enumerate(self) if hdu._data_loaded]
 
             hdulist.__file.close()
@@ -900,9 +955,9 @@ class HDUList(list, _Verify):
                 # Close all open mmaps to the data.  This is only necessary on
                 # Windows, which will not allow a file to be renamed or deleted
                 # until all handles to that file have been closed.
-                for idx, mmap, arr in mmaps:
-                    if mmap is not None:
-                        mmap.close()
+                for idx, map, arr in mmaps:
+                    if map is not None:
+                        map.base.close()
 
             os.remove(self.__file.name)
 
@@ -923,7 +978,7 @@ class HDUList(list, _Verify):
                 # Need to update the _file attribute and close any open mmaps
                 # on each HDU
                 if (hdu._data_loaded and
-                    _get_array_mmap(hdu.data) is not None):
+                    _get_array_memmap(hdu.data) is not None):
                     del hdu.data
                 hdu._file = ffo
 
@@ -968,8 +1023,7 @@ class HDUList(list, _Verify):
         self._resize = False
         self._truncate = False
         for hdu in self:
-            hdu._header._mod = False
-            hdu._header.ascard._mod = False
+            hdu._header._modified = False
             hdu._new = False
             hdu._file = ffo
 
@@ -986,11 +1040,8 @@ class HDUList(list, _Verify):
             # determine if any of the HDU is resized
             for hdu in self:
                 # Header:
-                # Add 1 to .ascard to include the END card
-                _nch80 = sum([card._ncards() for card in hdu._header.ascard])
-                _bytes = (_nch80+1) * Card.length
-                _bytes = _bytes + _pad_length(_bytes)
-                if _bytes != (hdu._datLoc - hdu._hdrLoc):
+                nbytes = len(str(hdu._header))
+                if nbytes != (hdu._datLoc - hdu._hdrLoc):
                     self._resize = True
                     self._truncate = False
                     if verbose:
@@ -1001,9 +1052,9 @@ class HDUList(list, _Verify):
                 if not hdu._data_loaded or hdu.data is None:
                     continue
 
-                _bytes = hdu.size()
-                _bytes = _bytes + _pad_length(_bytes)
-                if _bytes != hdu._datSpan:
+                nbytes = hdu.size
+                nbytes = nbytes + _pad_length(nbytes)
+                if nbytes != hdu._datSpan:
                     self._resize = True
                     self._truncate = False
                     if verbose:
@@ -1011,10 +1062,10 @@ class HDUList(list, _Verify):
                     break
 
             if self._truncate:
-               try:
-                   self.__file.truncate(hdu._datLoc + hdu._datSpan)
-               except IOError:
-                   self._resize = True
-               self._truncate = False
+                try:
+                    self.__file.truncate(hdu._datLoc + hdu._datSpan)
+                except IOError:
+                    self._resize = True
+                self._truncate = False
 
         return self._resize
