@@ -1,11 +1,10 @@
 import sys
 import numpy as np
 
-from pyfits.card import Card, CardList
 from pyfits.hdu.base import DELAYED, _ValidHDU, ExtensionHDU
 from pyfits.header import Header
-from pyfits.util import (_is_pseudo_unsigned, _unsigned_zero, _is_int,
-                         _pad_length, _normalize_slice, lazyproperty)
+from pyfits.util import _is_pseudo_unsigned, _unsigned_zero, _is_int, \
+                        _pad_length, _normalize_slice, lazyproperty
 
 class _ImageBaseHDU(_ValidHDU):
     """FITS image HDU base class.
@@ -26,13 +25,12 @@ class _ImageBaseHDU(_ValidHDU):
     """
 
     # mappings between FITS and numpy typecodes
-    # NumCode = {8:'int8', 16:'int16', 32:'int32', 64:'int64', -32:'float32', -64:'float64'}
-    # ImgCode = {'<i2':8, '<i4':16, '<i8':32, '<i16':64, '<f8':-32, '<f16':-64)
-    NumCode = {8:'uint8', 16:'int16', 32:'int32', 64:'int64', -32:'float32',
-               -64:'float64'}
-    ImgCode = {'uint8':8, 'int16':16, 'uint16':16, 'int32':32,
-               'uint32':32, 'int64':64, 'uint64':64,
-               'float32':-32, 'float64':-64}
+    # TODO: Maybe make these module-level constants instead...
+    NumCode = {8: 'uint8', 16: 'int16', 32: 'int32', 64: 'int64',
+               -32: 'float32', -64: 'float64'}
+    ImgCode = {'uint8': 8, 'int16': 16, 'uint16': 16, 'int32': 32,
+               'uint32': 32, 'int64': 64, 'uint64': 64, 'float32': -32,
+               'float64': -64}
 
     standard_keyword_comments = {
         'SIMPLE': 'conforms to FITS standard',
@@ -45,7 +43,7 @@ class _ImageBaseHDU(_ValidHDU):
     }
 
     def __init__(self, data=None, header=None, do_not_scale_image_data=False,
-                 uint=False, **kwargs):
+                 uint=False, scale_back=False, **kwargs):
         from pyfits.hdu.groups import GroupsHDU
 
         super(_ImageBaseHDU, self).__init__(data=data, header=header)
@@ -73,34 +71,37 @@ class _ImageBaseHDU(_ValidHDU):
             # PrimaryHDU and GroupsHDU subclasses
             # construct a list of cards of minimal header
             if isinstance(self, ExtensionHDU):
-                c0 = Card('XTENSION', 'IMAGE',
-                          self.standard_keyword_comments['XTENSION'])
+                c0 = ('XTENSION', 'IMAGE',
+                      self.standard_keyword_comments['XTENSION'])
             else:
-                c0 = Card('SIMPLE', True,
-                          self.standard_keyword_comments['SIMPLE'])
-            _list = CardList([
+                c0 = ('SIMPLE', True, self.standard_keyword_comments['SIMPLE'])
+            cards = [
                 c0,
-                Card('BITPIX',    8, self.standard_keyword_comments['BITPIX']),
-                Card('NAXIS',     0, self.standard_keyword_comments['NAXIS'])])
+                ('BITPIX',    8, self.standard_keyword_comments['BITPIX']),
+                ('NAXIS',     0, self.standard_keyword_comments['NAXIS'])]
 
             if isinstance(self, GroupsHDU):
-                _list.append(Card('GROUPS', True,
-                                  self.standard_keyword_comments['GROUPS']))
+                cards.append(('GROUPS', True,
+                             self.standard_keyword_comments['GROUPS']))
 
             if isinstance(self, (ExtensionHDU, GroupsHDU)):
-                _list.append(Card('PCOUNT',    0,
-                                  self.standard_keyword_comments['PCOUNT']))
-                _list.append(Card('GCOUNT',    1,
-                                  self.standard_keyword_comments['GCOUNT']))
+                cards.append(('PCOUNT',    0,
+                              self.standard_keyword_comments['PCOUNT']))
+                cards.append(('GCOUNT',    1,
+                              self.standard_keyword_comments['GCOUNT']))
 
             if header is not None:
-                hcopy = header.copy(strip=True)
-                _list.extend(hcopy.ascard)
+                orig = header.copy()
+                header = Header(cards)
+                header.extend(orig, strip=True, update=True, end=True)
+            else:
+                header = Header(cards)
 
-            self._header = Header(_list)
+            self._header = header
 
         self._do_not_scale_image_data = do_not_scale_image_data
         self._uint = uint
+        self._scale_back = scale_back
 
         if do_not_scale_image_data:
             self._bzero = 0
@@ -147,10 +148,6 @@ class _ImageBaseHDU(_ValidHDU):
 
         raise NotImplementedError
 
-    @property
-    def is_image(self):
-        return True
-
     @property
     def section(self):
         """
@@ -178,16 +175,17 @@ class _ImageBaseHDU(_ValidHDU):
         # Determine from the values read from the header
         return tuple(reversed(self._axes))
 
+    @property
     def header(self):
         return self._header
 
-    def _set_header(self, header):
+    @header.setter
+    def header(self, header):
         self._header = header
         self._modified = True
         self.update_header()
 
-    header = property(header, _set_header)
-
+    @lazyproperty
     def data(self):
         if len(self._axes) < 1:
             return
@@ -197,16 +195,11 @@ class _ImageBaseHDU(_ValidHDU):
 
         return data
 
-    def _set_data(self, data):
-        if 'data' in self.__dict__:
-            if self.__dict__['data'] is data:
-                return
-            else:
-                self._data_replaced = True
-        else:
-            self._data_replaced = True
-
-        if data is not None and not isinstance(data, np.ndarray):
+    @data.setter
+    def data(self, data):
+        self.__dict__['data'] = data
+        self._modified = True
+        if self.data is not None and not isinstance(data, np.ndarray):
             # Try to coerce the data into a numpy array--this will work, on
             # some level, for most objects
             try:
@@ -215,9 +208,6 @@ class _ImageBaseHDU(_ValidHDU):
                 raise TypeError('data object %r could not be coerced into an '
                                 'ndarray' % data)
 
-        self.__dict__['data'] = data
-        self._modified = True
-
         if isinstance(data, np.ndarray):
             self._bitpix = _ImageBaseHDU.ImgCode[data.dtype.name]
             self._orig_bitpix = self._bitpix
@@ -232,18 +222,12 @@ class _ImageBaseHDU(_ValidHDU):
 
         self.update_header()
 
-        # returning the data signals to lazyproperty that we've already handled
-        # setting self.__dict__['data']
-        return data
-
-    data = lazyproperty(data, _set_data)
-
     def update_header(self):
         """
         Update the header keywords to agree with the data.
         """
 
-        if not (self._modified or self._header._mod or
+        if not (self._modified or self._header._modified or
                 (self._data_loaded and self.shape != self.data.shape)):
             # Not likely that anything needs updating
             return
@@ -253,11 +237,11 @@ class _ImageBaseHDU(_ValidHDU):
         if 'BITPIX' not in self._header:
             bitpix_comment = self.standard_keyword_comments['BITPIX']
         else:
-            bitpix_comment = self._header.ascard['BITPIX'].comment
+            bitpix_comment = self._header.comments['BITPIX']
 
         # Update the BITPIX keyword and ensure it's in the correct
         # location in the header
-        self._header.update('BITPIX', self._bitpix, bitpix_comment, after=0)
+        self._header.set('BITPIX', self._bitpix, bitpix_comment, after=0)
 
         # If the data's shape has changed (this may have happened without our
         # noticing either via a direct update to the data.shape attribute) we
@@ -269,27 +253,31 @@ class _ImageBaseHDU(_ValidHDU):
         # Update the NAXIS keyword and ensure it's in the correct location in
         # the header
         if 'NAXIS' in self._header:
-            naxis_comment = self._header.ascard['NAXIS'].comment
+            naxis_comment = self._header.comments['NAXIS']
         else:
             naxis_comment = self.standard_keyword_comments['NAXIS']
-        self._header.update('NAXIS', len(self._axes), naxis_comment,
-                            after='BITPIX')
+        self._header.set('NAXIS', len(self._axes), naxis_comment,
+                         after='BITPIX')
 
+        # TODO: This routine is repeated in several different classes--it
+        # should probably be made available as a methond on all standard HDU
+        # types
         # add NAXISi if it does not exist
         for idx, axis in enumerate(self._axes):
-            try:
-                self._header['NAXIS'+ str(idx + 1)] = axis
-            except KeyError:
+            naxisn = 'NAXIS' + str(idx + 1)
+            if naxisn in self._header:
+                self._header[naxisn] = axis
+            else:
                 if (idx == 0):
                     after = 'NAXIS'
-                else :
+                else:
                     after = 'NAXIS' + str(idx)
-                self._header.update('naxis' + str(idx + 1), axis, after=after)
+                self._header.set(naxisn, axis, after=after)
 
         # delete extra NAXISi's
         for idx in range(len(self._axes) + 1, old_naxis + 1):
             try:
-                del self._header.ascard['NAXIS' + str(idx)]
+                del self._header['NAXIS' + str(idx)]
             except KeyError:
                 pass
 
@@ -304,7 +292,7 @@ class _ImageBaseHDU(_ValidHDU):
                     # Since _update_header_scale_info can, currently, be called
                     # *after* _prewriteto(), replace these with blank cards so
                     # the header size doesn't change
-                    self._header.add_blank()
+                    self._header.append()
                 except KeyError:
                     pass
 
@@ -317,7 +305,7 @@ class _ImageBaseHDU(_ValidHDU):
             self._bscale = 1
             self._bitpix = self._header['BITPIX']
 
-    def scale(self, type=None, option="old", bscale=1, bzero=0):
+    def scale(self, type=None, option='old', bscale=1, bzero=0):
         """
         Scale image data by using ``BSCALE``/``BZERO``.
 
@@ -355,7 +343,7 @@ class _ImageBaseHDU(_ValidHDU):
 
         # Determine how to scale the data
         # bscale and bzero takes priority
-        if (bscale != 1 or bzero !=0):
+        if (bscale != 1 or bzero != 0):
             _scale = bscale
             _zero = bzero
         else:
@@ -373,28 +361,36 @@ class _ImageBaseHDU(_ValidHDU):
 
                     if _type == np.uint8:  # uint8 case
                         _zero = min
-                        _scale = (max - min) / (2.**8 - 1)
+                        _scale = (max - min) / (2.0 ** 8 - 1)
                     else:
-                        _zero = (max + min) / 2.
+                        _zero = (max + min) / 2.0
 
                         # throw away -2^N
-                        _scale = (max - min) / (2.**(8*_type().itemsize) - 2)
+                        nbytes = 8 * _type().itemsize
+                        _scale = (max - min) / (2.0 ** nbytes - 2)
 
         # Do the scaling
         if _zero != 0:
-            self.data += -_zero # 0.9.6.3 to avoid out of range error for BZERO = +32768
-            self._header.update('BZERO', _zero)
+            # 0.9.6.3 to avoid out of range error for BZERO = +32768
+            self.data += -_zero
+            self._header['BZERO'] = _zero
         else:
-            del self._header['BZERO']
+            try:
+                del self._header['BZERO']
+            except KeyError:
+                pass
 
         if _scale and _scale != 1:
             self.data /= _scale
-            self._header.update('BSCALE', _scale)
+            self._header['BSCALE'] = _scale
         else:
-            del self._header['BSCALE']
+            try:
+                del self._header['BSCALE']
+            except KeyError:
+                pass
 
         if self.data.dtype.type != _type:
-            self.data = np.array(np.around(self.data), dtype=_type) #0.7.7.1
+            self.data = np.array(np.around(self.data), dtype=_type)
 
         # Update the BITPIX Card to match the data
         self._bitpix = _ImageBaseHDU.ImgCode[self.data.dtype.name]
@@ -417,6 +413,9 @@ class _ImageBaseHDU(_ValidHDU):
         return super(_ImageBaseHDU, self)._verify(option)
 
     def _prewriteto(self, checksum=False, inplace=False):
+        if self._scale_back:
+            self.scale(self.NumCode[self._orig_bitpix])
+
         self.update_header()
         if not inplace and not self._data_loaded:
             self._update_header_scale_info()
@@ -479,7 +478,6 @@ class _ImageBaseHDU(_ValidHDU):
         elif bitpix > 0:  # scale integers to Float32
             return np.dtype('float32')
 
-
     def _convert_pseudo_unsigned(self, data):
         """
         Handle "pseudo-unsigned" integers, if the user requested it.  Returns
@@ -494,10 +492,10 @@ class _ImageBaseHDU(_ValidHDU):
         # bool(dtype) is always False--have to explicitly compare to None; this
         # caused a fair amount of hair loss
         if dtype is not None and dtype.kind == 'u':
-        #    # Convert the input raw data into an unsigned integer array and
-        #    # then scale the data adjusting for the value of BZERO.  Note that
-        #    # we subtract the value of BZERO instead of adding because of the
-        #    # way numpy converts the raw signed array into an unsigned array.
+            # Convert the input raw data into an unsigned integer array and
+            # then scale the data adjusting for the value of BZERO.  Note that
+            # we subtract the value of BZERO instead of adding because of the
+            # way numpy converts the raw signed array into an unsigned array.
             bits = dtype.itemsize * 8
             data = np.array(data, dtype=dtype)
             data -= np.uint64(1 << (bits - 1))
@@ -512,7 +510,7 @@ class _ImageBaseHDU(_ValidHDU):
 
         code = _ImageBaseHDU.NumCode[self._orig_bitpix]
 
-        raw_data = self._file.readarray(offset=offset, dtype=code, shape=shape)
+        raw_data = self._get_raw_data(shape, code, offset)
         raw_data.dtype = raw_data.dtype.newbyteorder('>')
 
         if (self._orig_bzero == 0 and self._orig_bscale == 1 and
@@ -566,7 +564,7 @@ class _ImageBaseHDU(_ValidHDU):
         Summarize the HDU: name, dimensions, and formats.
         """
 
-        class_name  = self.__class__.__name__
+        class_name = self.__class__.__name__
 
         # if data is touched, use data info.
         if self._data_loaded:
@@ -623,7 +621,7 @@ class _ImageBaseHDU(_ValidHDU):
             # yet.  We can handle that in a generic manner so we do it in the
             # base class.  The other possibility is that there is no data at
             # all.  This can also be handled in a gereric manner.
-            return super(_ImageBaseHDU,self)._calculate_datasum(
+            return super(_ImageBaseHDU, self)._calculate_datasum(
                     blocking=blocking)
 
 
@@ -740,7 +738,7 @@ class Section(object):
                         if k == ns.start:
                             out = self[key1]
                         else:
-                            out = np.append(out,self[key1])
+                            out = np.append(out, self[key1])
 
                 # We have the data so break out of the loop.
                 break
@@ -757,7 +755,7 @@ class PrimaryHDU(_ImageBaseHDU):
     """
 
     def __init__(self, data=None, header=None, do_not_scale_image_data=False,
-                 uint=False):
+                 uint=False, scale_back=False):
         """
         Construct a primary HDU.
 
@@ -779,13 +777,20 @@ class PrimaryHDU(_ImageBaseHDU):
             central value and ``BSCALE == 1`` as unsigned integer
             data.  For example, `int16` data with ``BZERO = 32768``
             and ``BSCALE = 1`` would be treated as `uint16` data.
+
+        scale_back : bool, optional
+            If `True`, when saving changes to a file that contained scaled
+            image data, restore the data to the original type and reapply the
+            original BSCALE/BZERO values.  This could lead to loss of accuracy
+            if scaling back to integer values after performing floating point
+            operations on the data.
         """
 
         super(PrimaryHDU, self).__init__(
             data=data, header=header,
-            do_not_scale_image_data=do_not_scale_image_data, uint=uint)
-
-        self._name = 'PRIMARY'
+            do_not_scale_image_data=do_not_scale_image_data, uint=uint,
+            scale_back=scale_back)
+        self.name = 'PRIMARY'
         self._extver = 1
 
         # insert the keywords EXTEND
@@ -793,14 +798,14 @@ class PrimaryHDU(_ImageBaseHDU):
             dim = self._header['NAXIS']
             if dim == 0:
                 dim = ''
-            self._header.update('EXTEND', True, after='NAXIS' + str(dim))
+            self._header.set('EXTEND', True, after='NAXIS' + str(dim))
 
     @classmethod
     def match_header(cls, header):
-        card = header.ascard[0]
-        return card.key == 'SIMPLE' and \
-               ('GROUPS' not in header or header['GROUPS'] != True) and \
-               card.value == True
+        card = header.cards[0]
+        return (card.keyword == 'SIMPLE' and
+                ('GROUPS' not in header or header['GROUPS'] != True) and
+                card.value == True)
 
     def update_header(self):
         super(PrimaryHDU, self).update_header()
@@ -811,7 +816,7 @@ class PrimaryHDU(_ImageBaseHDU):
                 after = 'NAXIS' + str(len(self._axes))
             else:
                 after = 'NAXIS'
-            self._header.update('EXTEND', self._header['EXTEND'], after=after)
+            self._header.set('EXTEND', after=after)
 
     def _verify(self, option='warn'):
         errs = super(PrimaryHDU, self)._verify(option=option)
@@ -835,7 +840,7 @@ class ImageHDU(_ImageBaseHDU, ExtensionHDU):
     _extension = 'IMAGE'
 
     def __init__(self, data=None, header=None, name=None,
-                 do_not_scale_image_data=False, uint=False):
+                 do_not_scale_image_data=False, uint=False, scale_back=False):
         """
         Construct an image HDU.
 
@@ -861,6 +866,13 @@ class ImageHDU(_ImageBaseHDU, ExtensionHDU):
             central value and ``BSCALE == 1`` as unsigned integer
             data.  For example, `int16` data with ``BZERO = 32768``
             and ``BSCALE = 1`` would be treated as `uint16` data.
+
+        scale_back : bool, optional
+            If `True`, when saving changes to a file that contained scaled
+            image data, restore the data to the original type and reapply the
+            original BSCALE/BZERO values.  This could lead to loss of accuracy
+            if scaling back to integer values after performing floating point
+            operations on the data.
         """
 
         # This __init__ currently does nothing differently from the base class,
@@ -868,15 +880,16 @@ class ImageHDU(_ImageBaseHDU, ExtensionHDU):
 
         super(ImageHDU, self).__init__(
             data=data, header=header, name=name,
-            do_not_scale_image_data=do_not_scale_image_data, uint=uint)
+            do_not_scale_image_data=do_not_scale_image_data, uint=uint,
+            scale_back=scale_back)
 
     @classmethod
     def match_header(cls, header):
-        card = header.ascard[0]
+        card = header.cards[0]
         xtension = card.value
         if isinstance(xtension, basestring):
             xtension = xtension.rstrip()
-        return card.key == 'XTENSION' and xtension == cls._extension
+        return card.keyword == 'XTENSION' and xtension == cls._extension
 
     def _verify(self, option='warn'):
         """
@@ -892,8 +905,9 @@ class ImageHDU(_ImageBaseHDU, ExtensionHDU):
                        0, option, errs)
         return errs
 
+
 def _iswholeline(indx, naxis):
-    if isinstance(indx, (int, long,np.integer)):
+    if _is_int(indx):
         if indx >= 0 and indx < naxis:
             if naxis > 1:
                 return _SinglePoint(1, indx)
@@ -907,15 +921,15 @@ def _iswholeline(indx, naxis):
             return _WholeLine(naxis, 0)
         else:
             if indx.step == 1:
-                return _LineSlice(indx.stop-indx.start, indx.start)
+                return _LineSlice(indx.stop - indx.start, indx.start)
             else:
-                return _SteppedSlice((indx.stop-indx.start) // indx.step,
+                return _SteppedSlice((indx.stop - indx.start) // indx.step,
                                      indx.start)
     else:
         raise IndexError('Illegal index %s' % indx)
 
 
-class _KeyType:
+class _KeyType(object):
     def __init__(self, npts, offset):
         self.npts = npts
         self.offset = offset
@@ -939,7 +953,3 @@ class _LineSlice(_KeyType):
 
 class _SteppedSlice(_KeyType):
     pass
-
-
-
-
