@@ -1,11 +1,8 @@
-from __future__ import division  # confidence high
+from __future__ import division # confidence high
 from __future__ import with_statement
 
 import gzip
-import mmap
 import os
-import shutil
-import sys
 import warnings
 import zipfile
 
@@ -13,9 +10,9 @@ import numpy as np
 
 import pyfits
 from pyfits.convenience import _getext
-from pyfits.file import _File
+from pyfits.util import BytesIO
 from pyfits.tests import PyfitsTestCase
-from pyfits.tests.util import catch_warnings, BytesIO, ignore_warnings
+from pyfits.tests.util import catch_warnings, ignore_warnings
 
 from nose.tools import assert_equal, assert_raises, assert_true, assert_false
 
@@ -25,17 +22,13 @@ class TestCore(PyfitsTestCase):
         with pyfits.open(self.data('ascii.fits')) as f:
             pass
 
-    def test_missing_file(self):
-        assert_raises(IOError, pyfits.open, self.temp('does-not-exist.fits'))
-
     def test_naxisj_check(self):
         hdulist = pyfits.open(self.data('o4sp040b0_raw.fits'))
 
-        hdulist[1].header.update("NAXIS3", 500)
+        hdulist[1].header['NAXIS3'] = 500
 
         assert 'NAXIS3' in hdulist[1].header
-        with ignore_warnings():
-            hdulist.verify('fix')
+        hdulist.verify('silentfix')
         assert 'NAXIS3' not in hdulist[1].header
 
     def test_byteswap(self):
@@ -104,11 +97,13 @@ class TestCore(PyfitsTestCase):
 
         header = pyfits.Header()
         comment = 'number of bits per data pixel'
-        header.update('BITPIX', 16, comment)
+        header['BITPIX'] = (16, comment)
         assert 'BITPIX' in header
         assert_equal(header['BITPIX'], 16)
         assert_equal(header.ascard['BITPIX'].comment, comment)
 
+        # The new API doesn't support savecomment so leave this line here; at
+        # any rate good to have testing of the new API mixed with the old API
         header.update('BITPIX', 32, savecomment=True)
         # Make sure the value has been updated, but the comment was preserved
         assert_equal(header['BITPIX'], 32)
@@ -116,7 +111,7 @@ class TestCore(PyfitsTestCase):
 
         # The comment should still be preserved--savecomment only takes effect if
         # a new comment is also specified
-        header.update('BITPIX', 16)
+        header['BITPIX'] = 16
         assert_equal(header.ascard['BITPIX'].comment, comment)
         header.update('BITPIX', 16, 'foobarbaz', savecomment=True)
         assert_equal(header.ascard['BITPIX'].comment, comment)
@@ -157,7 +152,7 @@ class TestCore(PyfitsTestCase):
 
     def test_fix_invalid_keyword_value(self):
         hdu = pyfits.ImageHDU()
-        hdu.header.update('TESTKW', 'foo')
+        hdu.header['TESTKW'] = 'foo'
         errs = hdu.req_cards('TESTKW', None,
                              lambda v: v == 'foo', 'foo', 'ignore', [])
         assert_equal(len(errs), 0)
@@ -270,18 +265,51 @@ class TestCore(PyfitsTestCase):
         assert_equal(hdu.header['EXTNAME'], 'SCI')
 
         try:
-            pyfits.setExtensionNameCaseSensitive(True)
+            pyfits.EXTENSION_NAME_CASE_SENSITIVE = True
             hdu = pyfits.ImageHDU()
             hdu.name = 'sCi'
             assert_equal(hdu.name, 'sCi')
             assert_equal(hdu.header['EXTNAME'], 'sCi')
         finally:
-            pyfits.setExtensionNameCaseSensitive(False)
+            pyfits.EXTENSION_NAME_CASE_SENSITIVE = False
 
         hdu.name = 'sCi'
         assert_equal(hdu.name, 'SCI')
         assert_equal(hdu.header['EXTNAME'], 'SCI')
 
+    def test_hdu_fromstring(self):
+        """
+        Tests creating a fully-formed HDU object from a string containing the
+        bytes of the HDU.
+        """
+
+        dat = open(self.data('test0.fits'), 'rb').read()
+
+        offset = 0
+        with pyfits.open(self.data('test0.fits')) as hdul:
+            hdulen = hdul[0]._datLoc + hdul[0]._datSpan
+            hdu = pyfits.PrimaryHDU.fromstring(dat[:hdulen])
+            assert_true(isinstance(hdu, pyfits.PrimaryHDU))
+            assert_equal(hdul[0].header, hdu.header)
+            assert_true(hdu.data is None)
+
+        hdu.header['TEST'] = 'TEST'
+        hdu.writeto(self.temp('test.fits'))
+        with pyfits.open(self.temp('test.fits')) as hdul:
+            assert_true(isinstance(hdu, pyfits.PrimaryHDU))
+            assert_equal(hdul[0].header[:-1], hdu.header[:-1])
+            assert_equal(hdul[0].header['TEST'], 'TEST')
+            assert_true(hdu.data is None)
+
+        with pyfits.open(self.data('test0.fits'))as hdul:
+            for ext_hdu in hdul[1:]:
+                offset += hdulen
+                hdulen = len(str(ext_hdu.header)) + ext_hdu._datSpan
+                hdu = pyfits.ImageHDU.fromstring(dat[offset:offset + hdulen])
+                assert_true(isinstance(hdu, pyfits.ImageHDU))
+                assert_equal(ext_hdu.header, hdu.header)
+                assert_true((ext_hdu.data == hdu.data).all())
+
     def test_nonstandard_hdu(self):
         """
         Regression test for #157.  Tests that "Nonstandard" HDUs with SIMPLE =
@@ -339,26 +367,38 @@ class TestConvenienceFunctions(PyfitsTestCase):
 
 
 class TestFileFunctions(PyfitsTestCase):
-    """Tests various basic I/O operations, specifically in the
-    pyfits.file._File class.
+    """
+    Tests various basic I/O operations, specifically in the pyfits.file._File
+    class.
     """
 
     def test_open_gzipped(self):
-        assert_equal(len(pyfits.open(self._make_gzip_file())), 5)
+        with ignore_warnings():
+            assert_equal(len(pyfits.open(self._make_gzip_file())), 5)
 
     def test_detect_gzipped(self):
         """Test detection of a gzip file when the extension is not .gz."""
 
-        assert_equal(len(pyfits.open(self._make_gzip_file('test0.fz'))), 5)
+        with ignore_warnings():
+            assert_equal(len(pyfits.open(self._make_gzip_file('test0.fz'))), 5)
+
+    def test_open_gzipped_writeable(self):
+        """Opening gzipped files in a writeable mode should fail."""
+
+        gf = self._make_gzip_file()
+        assert_raises(IOError, pyfits.open, gf, 'update')
+        assert_raises(IOError, pyfits.open, gf, 'append')
 
     def test_open_zipped(self):
-        assert_equal(len(pyfits.open(self._make_zip_file())), 5)
+        with ignore_warnings():
+            assert_equal(len(pyfits.open(self._make_zip_file())), 5)
 
     def test_detect_zipped(self):
         """Test detection of a zip file when the extension is not .zip."""
 
         zf = self._make_zip_file(filename='test0.fz')
-        assert_equal(len(pyfits.open(zf)), 5)
+        with ignore_warnings():
+            assert_equal(len(pyfits.open(zf)), 5)
 
     def test_open_zipped_writeable(self):
         """Opening zipped files in a writeable mode should fail."""
@@ -367,8 +407,9 @@ class TestFileFunctions(PyfitsTestCase):
         assert_raises(IOError, pyfits.open, zf, 'update')
         assert_raises(IOError, pyfits.open, zf, 'append')
 
-    def test_open_multipe_member_zipfile(self):
-        """Opening zip files containing more than one member files should fail
+    def test_open_multiple_member_zipfile(self):
+        """
+        Opening zip files containing more than one member files should fail
         as there's no obvious way to specify which file is the FITS file to
         read.
         """
@@ -402,17 +443,6 @@ class TestFileFunctions(PyfitsTestCase):
         finally:
             gf.close()
 
-    def test_open_gzip_file_for_writing(self):
-        """Regression test for #195."""
-
-        gf = self._make_gzip_file()
-        with pyfits.open(gf, mode='update') as h:
-            h[0].header['EXPFLAG'] = 'ABNORMAL'
-        with pyfits.open(gf) as h:
-            # Just to make sur ethe update worked; if updates work
-            # normal writes should work too...
-            assert_equal(h[0].header['EXPFLAG'], 'ABNORMAL')
-
     def test_read_file_like_object(self):
         """Test reading a FITS file from a file-like object."""
 
@@ -420,7 +450,8 @@ class TestFileFunctions(PyfitsTestCase):
         with open(self.data('test0.fits'), 'rb') as f:
             filelike.write(f.read())
         filelike.seek(0)
-        assert_equal(len(pyfits.open(filelike)), 5)
+        with ignore_warnings():
+            assert_equal(len(pyfits.open(filelike)), 5)
 
     def test_updated_file_permissions(self):
         """
@@ -442,45 +473,6 @@ class TestFileFunctions(PyfitsTestCase):
 
         assert_equal(old_mode, os.stat(filename).st_mode)
 
-    if sys.version_info[:2] > (2, 5):
-        # After a fair bit of experimentation I found that it's more difficult
-        # than it's worth to wrap mmap in Python 2.5.
-        def test_mmap_unwriteable(self):
-            """Regression test for
-            https://github.com/astropy/astropy/issues/968
-
-            Temporarily patches mmap.mmap to exhibit platform-specific bad
-            behavior.
-            """
-
-            class MockMmap(mmap.mmap):
-                def flush(self):
-                    raise mmap.error('flush is broken on this platform')
-
-            old_mmap = mmap.mmap
-            mmap.mmap = MockMmap
-
-            # Force the mmap test to be rerun
-            _File._mmap_available = None
-
-            try:
-                # TODO: Use self.copy_file once it's merged into Astropy
-                shutil.copy(self.data('test0.fits'), self.temp('test0.fits'))
-                with catch_warnings(record=True) as w:
-                    with pyfits.open(self.temp('test0.fits'), mode='update',
-                                     memmap=True) as h:
-                        h[1].data[0, 0] = 999
-
-                    assert len(w) == 1
-                    assert 'mmap.flush is unavailable' in str(w[0].message)
-
-                # Double check that writing without mmap still worked
-                with pyfits.open(self.temp('test0.fits')) as h:
-                    assert h[1].data[0, 0] == 999
-            finally:
-                mmap.mmap = old_mmap
-                _File._mmap_available = None
-
     def _make_gzip_file(self, filename='test0.fits.gz'):
         gzfile = self.temp(filename)
         with open(self.data('test0.fits'), 'rb') as f:
@@ -503,12 +495,13 @@ class TestStreamingFunctions(PyfitsTestCase):
 
     def test_streaming_hdu(self):
         shdu = self._make_streaming_hdu(self.temp('new.fits'))
-        assert_true(isinstance(shdu.size(), int))
-        assert_equal(shdu.size(), 100)
+        assert_true(isinstance(shdu.size, int))
+        assert_equal(shdu.size, 100)
 
     def test_streaming_hdu_file_wrong_mode(self):
-        """Test that streaming an HDU to a file opened in the wrong mode
-        fails as expected.
+        """
+        Test that streaming an HDU to a file opened in the wrong mode fails as
+        expected.
         """
 
         with open(self.temp('new.fits'), 'wb') as f:
@@ -523,7 +516,7 @@ class TestStreamingFunctions(PyfitsTestCase):
             shdu = self._make_streaming_hdu(f)
             shdu.write(arr)
             assert_true(shdu.writecomplete)
-            assert_equal(shdu.size(), 100)
+            assert_equal(shdu.size, 100)
         hdul = pyfits.open(self.temp('new.fits'))
         assert_equal(len(hdul), 1)
         assert_true((hdul[0].data == arr).all())
@@ -537,7 +530,7 @@ class TestStreamingFunctions(PyfitsTestCase):
         shdu = self._make_streaming_hdu(sf)
         shdu.write(arr)
         assert_true(shdu.writecomplete)
-        assert_equal(shdu.size(), 100)
+        assert_equal(shdu.size, 100)
 
         sf.seek(0)
         hdul = pyfits.open(sf)
@@ -554,26 +547,12 @@ class TestStreamingFunctions(PyfitsTestCase):
             shdu = self._make_streaming_hdu(f)
             shdu.write(arr)
 
-    def test_fix_invalid_extname(self):
-        phdu = pyfits.PrimaryHDU()
-        ihdu = pyfits.ImageHDU()
-        ihdu.header.update('EXTNAME', 12345678)
-        hdul = pyfits.HDUList([phdu, ihdu])
-
-        assert_raises(pyfits.VerifyError, hdul.writeto, self.temp('temp.fits'),
-                      output_verify='exception')
-        with ignore_warnings():
-            hdul.writeto(self.temp('temp.fits'), output_verify='fix')
-        with pyfits.open(self.temp('temp.fits')):
-            assert_equal(hdul[1].name, '12345678')
-            assert_equal(hdul[1].header['EXTNAME'], '12345678')
-
     def _make_streaming_hdu(self, fileobj):
         hd = pyfits.Header()
-        hd.update('SIMPLE', True, 'conforms to FITS standard')
-        hd.update('BITPIX', 32, 'array data type')
-        hd.update('NAXIS', 2, 'number of array dimensions')
-        hd.update('NAXIS1', 5)
-        hd.update('NAXIS2', 5)
-        hd.update('EXTEND', True)
+        hd['SIMPLE'] = (True, 'conforms to FITS standard')
+        hd['BITPIX'] = (32, 'array data type')
+        hd['NAXIS'] = (2, 'number of array dimensions')
+        hd['NAXIS1'] = 5
+        hd['NAXIS2'] = 5
+        hd['EXTEND'] = True
         return pyfits.StreamingHDU(fileobj, hd)
