@@ -1,14 +1,17 @@
-from __future__ import division # confidence high
-from __future__ import with_statement
+from __future__ import division, with_statement  # confidence high
 
+import glob
 import os
+import shutil
+import sys
 
 import numpy as np
 
 import pyfits
+from pyfits.util import BytesIO
 from pyfits.verify import VerifyError
 from pyfits.tests import PyfitsTestCase
-from pyfits.tests.util import catch_warnings, BytesIO, ignore_warnings
+from pyfits.tests.util import catch_warnings, ignore_warnings
 
 from nose.tools import assert_equal, assert_raises, assert_true
 
@@ -33,7 +36,7 @@ class TestHDUListFunctions(PyfitsTestCase):
 
         def test_fileinfo(**kwargs):
             assert_equal(res['datSpan'], kwargs.get('datSpan', 2880))
-            assert_equal(res['resized'], kwargs.get('resized', 0))
+            assert_equal(res['resized'], kwargs.get('resized', False))
             assert_equal(res['filename'], self.data('checksum.fits'))
             assert_equal(res['datLoc'], kwargs.get('datLoc', 8640))
             assert_equal(res['hdrLoc'], kwargs.get('hdrLoc', 0))
@@ -46,10 +49,10 @@ class TestHDUListFunctions(PyfitsTestCase):
         hdul.insert(1, hdu)
 
         res = hdul.fileinfo(0)
-        test_fileinfo(resized=1)
+        test_fileinfo(resized=True)
 
         res = hdul.fileinfo(1)
-        test_fileinfo(datSpan=None, resized=1, datLoc=None, hdrLoc=None)
+        test_fileinfo(datSpan=None, resized=True, datLoc=None, hdrLoc=None)
 
         res = hdul.fileinfo(2)
         test_fileinfo(resized=1, datLoc=17280, hdrLoc=11520)
@@ -187,7 +190,7 @@ class TestHDUListFunctions(PyfitsTestCase):
         """Tests inserting a Simple ImageHDU to an empty HDUList."""
 
         hdul = pyfits.HDUList()
-        hdu = pyfits.ImageHDU(np.arange(100,dtype=np.int32))
+        hdu = pyfits.ImageHDU(np.arange(100, dtype=np.int32))
         hdul.insert(0, hdu)
 
         info = [(0, 'PRIMARY', 'PrimaryHDU', 4, (100,), 'int32', '')]
@@ -275,7 +278,7 @@ class TestHDUListFunctions(PyfitsTestCase):
         hdul.insert(0, hdu)
         hdu = pyfits.GroupsHDU()
 
-        assert_raises(ValueError, hdul.insert, hdul, 1, hdu)
+        assert_raises(ValueError, hdul.insert, 1, hdu)
 
         info = [(0, 'PRIMARY', 'GroupsHDU', 8, (), 'uint8',
                  '1 Groups  0 Parameters'),
@@ -300,7 +303,7 @@ class TestHDUListFunctions(PyfitsTestCase):
         hdu = pyfits.GroupsHDU()
         hdul.insert(0, hdu)
 
-        assert_raises(ValueError, hdul.insert, hdul, 0, hdu)
+        assert_raises(ValueError, hdul.insert, 0, hdu)
 
     def test_insert_extension_to_primary_in_non_empty_list(self):
         # Tests inserting a Simple ExtensionHDU to a non-empty HDUList.
@@ -429,7 +432,7 @@ class TestHDUListFunctions(PyfitsTestCase):
 
         oldmtime = os.stat(self.data('test0.fits')).st_mtime
         hdul = pyfits.open(self.data('test0.fits'))
-        hdul[0].header.update('FOO', 'BAR')
+        hdul[0].header['FOO'] = 'BAR'
         with catch_warnings(record=True) as w:
             hdul.flush()
             assert_equal(len(w), 1)
@@ -479,7 +482,7 @@ class TestHDUListFunctions(PyfitsTestCase):
                            do_not_scale_image_data=True)
         info = hdul.info(output=False)
         hdul.writeto(self.temp('temp.fits'))
-        with open(self.temp('temp.fits'),'ab') as f:
+        with open(self.temp('temp.fits'), 'ab') as f:
             f.seek(0, os.SEEK_END)
             f.write('\0'.encode('latin1') * 2880)
         with ignore_warnings():
@@ -498,7 +501,7 @@ class TestHDUListFunctions(PyfitsTestCase):
         hdu.writeto(self.temp('temp.fits'))
 
         # Figure out where the header padding begins and fill it with nulls
-        end_card_pos = len(repr(hdu.header.ascard))
+        end_card_pos = str(hdu.header).index('END' + ' ' * 77)
         padding_start = end_card_pos + 80
         padding_len = 2880 - padding_start
         with open(self.temp('temp.fits'), 'r+b') as f:
@@ -511,8 +514,7 @@ class TestHDUListFunctions(PyfitsTestCase):
                             str(w[0].message))
                 assert_equal(len(w), 1)
                 assert_equal(len(hdul), 1)
-                assert_equal(repr(hdul[0].header.ascard),
-                             repr(hdu.header.ascard))
+                assert_equal(str(hdul[0].header), str(hdu.header))
                 assert_true((hdul[0].data == a).all())
 
     def test_update_with_truncated_header(self):
@@ -526,7 +528,7 @@ class TestHDUListFunctions(PyfitsTestCase):
         hdu = pyfits.PrimaryHDU(data=data)
         idx = 1
         while len(hdu.header) < 34:
-            hdu.header.update('TEST%d' % idx, idx)
+            hdu.header['TEST%d' % idx] = idx
             idx += 1
         hdu.writeto(self.temp('temp.fits'), checksum=True)
 
@@ -540,37 +542,37 @@ class TestHDUListFunctions(PyfitsTestCase):
     def test_update_resized_header(self):
         """
         Test saving updates to a file where the header is one block smaller
-        than before, and in the case where the header is one block larger than
+        than before, and in the case where the heade ris one block larger than
         before.
         """
 
         data = np.arange(100)
         hdu = pyfits.PrimaryHDU(data=data)
         idx = 1
-        while len(hdu.header) <= 35:
-            hdu.header.update('TEST%d' % idx, idx)
+        while len(str(hdu.header)) <= 2880:
+            hdu.header['TEST%d' % idx] = idx
             idx += 1
         orig_header = hdu.header.copy()
         hdu.writeto(self.temp('temp.fits'))
 
         with pyfits.open(self.temp('temp.fits'), mode='update') as hdul:
-            while len(hdul[0].header) >= 36:
+            while len(str(hdul[0].header)) > 2880:
                 del hdul[0].header[-1]
 
         with pyfits.open(self.temp('temp.fits')) as hdul:
-            assert_equal(str(hdul[0].header.ascard), str(orig_header[:-1]))
+            assert_equal(hdul[0].header, orig_header[:-1])
             assert_true((hdul[0].data == data).all())
 
         with pyfits.open(self.temp('temp.fits'), mode='update') as hdul:
             idx = 101
-            while len(hdul[0].header) < 36 * 2:
-                hdul[0].header.update('TEST%d' % idx, idx)
+            while len(str(hdul[0].header)) <= 2880 * 2:
+                hdul[0].header['TEST%d' % idx] = idx
                 idx += 1
             # Touch something in the data too so that it has to be rewritten
             hdul[0].data[0] = 27
 
         with pyfits.open(self.temp('temp.fits')) as hdul:
-            assert_equal(str(hdul[0].header[:-37]), str(orig_header[:-1]))
+            assert_equal(hdul[0].header[:-37], orig_header[:-1])
             assert_equal(hdul[0].data[0], 27)
             assert_true((hdul[0].data[1:] == data[1:]).all())
 
@@ -586,13 +588,16 @@ class TestHDUListFunctions(PyfitsTestCase):
         data2 = np.arange(100) + 100
         phdu = pyfits.PrimaryHDU(data=data1)
         hdu = pyfits.ImageHDU(data=data2)
+
         phdu.writeto(self.temp('temp.fits'))
+
         with pyfits.open(self.temp('temp.fits'), mode='append') as hdul:
             hdul.append(hdu)
+
         with pyfits.open(self.temp('temp.fits'), mode='update') as hdul:
             idx = 1
-            while len(hdul[0].header) < 36 * 2:
-                hdul[0].header.update('TEST%d' % idx, idx)
+            while len(str(hdul[0].header)) <= 2880 * 2:
+                hdul[0].header['TEST%d' % idx] = idx
                 idx += 1
             hdul.flush()
             hdul.append(hdu)
@@ -603,82 +608,74 @@ class TestHDUListFunctions(PyfitsTestCase):
             assert_true((hdul[1].data == data2).all())
             assert_true((hdul[2].data == data2).all())
 
-    def test_replace_mmap_data(self):
-        """Regression test for
-        https://github.com/spacetelescope/PyFITS/issues/25
-
-        Replacing the mmap'd data of one file with mmap'd data from a
-        different file should work.
+    def test_hdul_fromstring(self):
+        """
+        Test creating the HDUList structure in memory from a string containing
+        an entire FITS file.  This is similar to test_hdu_fromstring but for an
+        entire multi-extension FITS file at once.
         """
 
-        arr_a = np.arange(10)
-        arr_b = arr_a * 2
+        # Tests HDUList.fromstring for all of PyFITS' built in test files
+        def test_fromstring(filename):
+            with pyfits.open(self.data(filename)) as hdul:
+                orig_info = hdul.info(output=False)
+                with open(self.data(filename), 'rb') as f:
+                    dat = f.read()
 
-        def test(mmap_a, mmap_b):
-            hdu_a = pyfits.PrimaryHDU(data=arr_a)
-            hdu_a.writeto(self.temp('test_a.fits'), clobber=True)
-            hdu_b = pyfits.PrimaryHDU(data=arr_b)
-            hdu_b.writeto(self.temp('test_b.fits'), clobber=True)
+                hdul2 = pyfits.HDUList.fromstring(dat)
 
-            hdul_a = pyfits.open(self.temp('test_a.fits'), mode='update',
-                                 memmap=mmap_a)
-            hdul_b = pyfits.open(self.temp('test_b.fits'), memmap=mmap_b)
-            hdul_a[0].data = hdul_b[0].data
-            hdul_a.close()
-            hdul_b.close()
+                assert_equal(orig_info, hdul2.info(output=False))
+                for idx in range(len(hdul)):
+                    assert_equal(hdul[idx].header, hdul2[idx].header)
+                    if  hdul[idx].data is None or hdul2[idx].data is None:
+                        assert_equal(hdul[idx].data, hdul2[idx].data)
+                    elif (hdul[idx].data.dtype.fields and
+                          hdul2[idx].data.dtype.fields):
+                        # Compare tables
+                        for n in hdul[idx].data.names:
+                            c1 = hdul[idx].data[n]
+                            c2 = hdul2[idx].data[n]
+                            assert_true((c1 == c2).all())
+                    else:
+                        assert_true((hdul[idx].data == hdul2[idx].data).all())
 
-            hdul_a = pyfits.open(self.temp('test_a.fits'))
+        for filename in glob.glob(os.path.join(self.data_dir, '*.fits')):
+            if sys.platform == 'win32' and filename == 'zerowidth.fits':
+                # Running this test on this file causes a crash in some
+                # versions of Numpy on Windows.  See PyFITS ticket #174
+                continue
+            test_fromstring(os.path.join(self.data_dir, filename))
 
-            assert_true(np.all(hdul_a[0].data == arr_b))
+        # Test that creating an HDUList from something silly raises a TypeError
+        assert_raises(TypeError, pyfits.HDUList.fromstring, ['a', 'b', 'c'])
+
+    def test_save_backup(self):
+        """Test for #121--save backup of file before flushing changes."""
+
+        shutil.copy(self.data('scale.fits'), self.temp('scale.fits'))
 
         with ignore_warnings():
-            test(True, True)
+            with pyfits.open(self.temp('scale.fits'), mode='update',
+                             save_backup=True) as hdul:
+                # Make some changes to the original file to force its header
+                # and data to be rewritten
+                hdul[0].header['TEST'] = 'TEST'
+                hdul[0].data[0] = 0
 
-            # Repeat the same test but this time don't mmap A
-            test(False, True)
-
-            # Finally, without mmaping B
-            test(True, False)
-
-
-    def test_replace_mmap_data_2(self):
-        """Regression test for
-        https://github.com/spacetelescope/PyFITS/issues/25
-
-        Replacing the mmap'd data of one file with mmap'd data from a
-        different file should work.  Like test_replace_mmap_data but with
-        table data instead of image data.
-        """
-
-        arr_a = np.arange(10)
-        arr_b = arr_a * 2
-
-        def test(mmap_a, mmap_b):
-            col_a = pyfits.Column(name='a', format='J', array=arr_a)
-            col_b = pyfits.Column(name='b', format='J', array=arr_b)
-            hdu_a = pyfits.new_table([col_a])
-            hdu_a.writeto(self.temp('test_a.fits'), clobber=True)
-            hdu_b = pyfits.new_table([col_b])
-            hdu_b.writeto(self.temp('test_b.fits'), clobber=True)
-
-            hdul_a = pyfits.open(self.temp('test_a.fits'), mode='update',
-                                 memmap=mmap_a)
-            hdul_b = pyfits.open(self.temp('test_b.fits'), memmap=mmap_b)
-            hdul_a[1].data = hdul_b[1].data
-            hdul_a.close()
-            hdul_b.close()
-
-            hdul_a = pyfits.open(self.temp('test_a.fits'))
-
-            assert_true('b' in hdul_a[1].columns.names)
-            assert_true('a' not in hdul_a[1].columns.names)
-            assert_true(np.all(hdul_a[1].data['b'] == arr_b))
+        assert_true(os.path.exists(self.temp('scale.fits.bak')))
+        with pyfits.open(self.data('scale.fits'),
+                         do_not_scale_image_data=True) as hdul1:
+            with pyfits.open(self.temp('scale.fits.bak'),
+                             do_not_scale_image_data=True) as hdul2:
+                assert_equal(hdul1[0].header, hdul2[0].header)
+                assert_true((hdul1[0].data == hdul2[0].data).all())
 
         with ignore_warnings():
-            test(True, True)
+            with pyfits.open(self.temp('scale.fits'), mode='update',
+                             save_backup=True) as hdul:
+                # One more time to see if multiple backups are made
+                hdul[0].header['TEST2'] = 'TEST'
+                hdul[0].data[0] = 1
 
-            # Repeat the same test but this time don't mmap A
-            test(False, True)
-
-            # Finally, without mmaping B
-            test(True, False)
+        assert_true(os.path.exists(self.temp('scale.fits.bak')))
+        assert_true(os.path.exists(self.temp('scale.fits.bak.1')))
