@@ -1,218 +1,35 @@
-from __future__ import division
-
+import __builtin__
 import functools
 import itertools
-import mmap
 import os
-import platform
 import signal
 import sys
 import tempfile
+import textwrap
 import threading
 import warnings
 
+try:
+    try:
+        from cStringIO import StringIO
+    except ImportError:
+        from StringIO import StringIO
+except ImportError:
+    from io import StringIO
+
+try:
+    from io import BytesIO
+except ImportError:
+    BytesIO = StringIO
+
 import numpy as np
 
-
-__all__ = ['Extendable', 'register_extension', 'register_extensions',
-           'unregister_extensions']
+from numpy import memmap as Memmap
 
 
 BLOCK_SIZE = 2880  # the FITS block size
 
 
-# TODO: I'm somewhat of the opinion that this should go in pyfits.core, but for
-# now that would create too much complication with imports (as many modules
-# need to use this).  Eventually all the intra-package imports will be removed
-# from pyfits.core, simplifying matters.  But for now they remain for
-# backwards-compatibility.
-class Extendable(type):
-    _extensions = {}
-
-    def __call__(cls, *args, **kwargs):
-        if cls in cls._extensions:
-            cls = cls._extensions[cls]
-        self = cls.__new__(cls, *args, **kwargs)
-        self.__init__(*args, **kwargs)
-        return self
-
-# TODO: Fix this in Python3--commented out in the meantime
-#    def __getattribute__(cls, attr):
-#        orig_cls = cls
-#        if attr != '_extensions' and cls in cls._extensions:
-#            cls = cls._extensions[cls]
-#        return super(Extendable, cls).__getattribute__(attr)
-
-    @classmethod
-    def register_extension(cls, extension, extends=None, silent=False):
-        """
-        Register an extension class.  This class will be used in all future
-        instances of the class it extends.
-
-        By default, the class it extends
-        will automatically be its immediate superclass.  In
-        multiple-inheritence cases, the left-most superclass is used.  In other
-        words, the first class in the extension's MRO (after itself).
-
-        Parameters
-        ----------
-        extension : class
-            The class object of a superclass of an Extendable class (that is,
-            any class with the `Extendable` metaclass
-
-        extends : class, optional
-            Override the default behavior of extending the immediate
-            superclass.  Either a single class to extend may be specified, or
-            an iterable of classes.  The classes to extend must still have the
-            `Extendable` metaclass.
-
-        silent : bool, optional
-            If `True`, does not output any warnings
-        """
-
-        # If the extension class itself is not Extendable then we know it's no
-        # good, since it has to be a sublcass of an Extendable.
-        if not isinstance(extension, Extendable):
-            raise TypeError("Class '%s' is not a subclass of an Extendable "
-                            "class." % extension.__name__)
-
-        if extends:
-            try:
-                extends = iter(extends)
-            except TypeError:
-                extends = [extends]
-        else:
-            extends = [extension.mro()[1]]
-
-        # Don't allow the extension to be registered if *any* of the classes it
-        # extends are not Extendable
-        for c in extends:
-            if not isinstance(c, Extendable):
-                raise TypeError("Class '%s' is not an Extendable class."
-                                % c.__name__)
-
-        for c in extends:
-            if c in cls._extensions:
-                if not silent:
-                    warnings.warn(
-                        "Extension '%s' for '%s' being replaced with '%s'."
-                        % (cls._extensions[c].__name__, c.__name__,
-                           extension.__name__))
-                # This class has already been extended by a different class, so
-                # first we need to undo that
-                cls._unextend_subclasses(c, extension)
-            # It's imperative that _extend_subclasses is called first;
-            # otherwise the extension will override c.__subclasses__!
-            cls._extend_subclasses(c, extension)
-            cls._extensions[c] = extension
-
-    @classmethod
-    def register_extensions(cls, extensions, silent=False):
-        """
-        Register multiple extensions at once from a dict mapping extensions to
-        the classes they extend.
-        """
-
-        for k, v in extensions.iteritems():
-            if not isinstance(k, Extendable):
-                raise TypeError("Extension class '%s' is not a subclass of "
-                                "an Extendable class." % k.__name__)
-            if not isinstance(v, Extendable):
-                raise TypeError("Class '%s' is not an Extendable class.")
-
-        for k, v in extensions.iteritems():
-            if not silent and v in cls._extensions:
-                warnings.showwarning(
-                    "Extension '%s' for '%s' being replaced with '%s'."
-                    % (cls._extensions[v].__name__, v.__name__, k.__name__))
-            cls._extend_subclasses(v, k)
-            cls._extensions[v] = k
-
-    @classmethod
-    def unregister_extensions(cls, extensions):
-        """
-        Remove one or more extension classes from the extension registry.
-
-        If the class is not in the registry this is silently ignored.
-        """
-
-        try:
-            extensions = set(extensions)
-        except TypeError:
-            extensions = set([extensions])
-
-        for k, v in cls._extensions.items():
-            if v in extensions:
-                del cls._extensions[k]
-                cls._unextend_subclasses(k, v)
-
-    @classmethod
-    def _extend_subclasses(cls, extendable, extension):
-        for s in extendable.__subclasses__():
-            if s is extension:
-                # Don't want the extension to have itself as a base
-                continue
-            bases = list(s.__bases__)
-            idx = bases.index(extendable)
-            bases = bases[:idx] + [extension] + bases[idx + 1:]
-            s.__bases__ = tuple(bases)
-
-    @classmethod
-    def _unextend_subclasses(cls, extendable, extension):
-        # Since the subclasses' bases were changed, they will now be listed as
-        # subclasses of the extension
-        for s in extension.__subclasses__():
-            if s is extension:
-                continue
-            bases = list(s.__bases__)
-            idx = bases.index(extension)
-            bases = bases[:idx] + [extendable] + bases[idx + 1:]
-            s.__bases__ = tuple(bases)
-
-# Some shortcuts
-register_extension = Extendable.register_extension
-register_extensions = Extendable.register_extensions
-# unregister_extension is currently the same as just unregister_extensions (it
-# won't balk if you still try to pass more than one)--it's just provided for
-# symmetry's sake
-unregister_extension = Extendable.unregister_extensions
-unregister_extensions = Extendable.unregister_extensions
-
-
-def _with_extensions(func):
-    """
-    This decorator exists mainly to support use of the new extension system in
-    functions that still have a classExtensions keyword argument (which should
-    be deprecated).
-
-    This registers the extensions passed in classExtensions and unregisters
-    them when the function exits.  It should be clear that any objects that
-    persist after the function exits will still use the extension classes they
-    were created from.
-    """
-
-    @functools.wraps(func)
-    def _with_extensions_wrapper(*args, **kwargs):
-        extension_classes = []
-        if 'classExtensions' in kwargs:
-            extensions = kwargs['classExtensions']
-            warnings.warn('The classExtensions argument is deprecated.  '
-                          'Instead call pyfits.register_extensions(%s) once '
-                          'before any code that uses those extensions.'
-                          % repr(extensions), DeprecationWarning)
-            if extensions:
-                register_extensions(extensions)
-                extension_classes = extensions.values()
-            del kwargs['classExtensions']
-        try:
-            return func(*args, **kwargs)
-        finally:
-            if extension_classes:
-                unregister_extensions(extension_classes)
-
-    return _with_extensions_wrapper
-
-
 def itersubclasses(cls, _seen=None):
     """
     itersubclasses(cls)
@@ -243,10 +60,11 @@ def itersubclasses(cls, _seen=None):
     if not isinstance(cls, type):
         raise TypeError('itersubclasses must be called with '
                         'new-style classes, not %.100r' % cls)
-    if _seen is None: _seen = set()
+    if _seen is None:
+        _seen = set()
     try:
         subs = cls.__subclasses__()
-    except TypeError: # fails only when cls is type
+    except TypeError:  # fails only when cls is type
         subs = cls.__subclasses__(cls)
     for sub in subs:
         if sub not in _seen:
@@ -264,10 +82,14 @@ class lazyproperty(object):
     http://code.activestate.com/recipes/363602-lazy-property-evaluation
     """
 
-    def __init__(self, fget, fset=None, fdel=None):
+    def __init__(self, fget, fset=None, fdel=None, doc=None):
         self._fget = fget
         self._fset = fset
         self._fdel = fdel
+        if doc is None:
+            self.__doc__ = fget.__doc__
+        else:
+            self.__doc__ = doc
 
     def __get__(self, obj, owner=None):
         if obj is None:
@@ -281,16 +103,9 @@ class lazyproperty(object):
             return obj.__dict__[key]
 
     def __set__(self, obj, val):
-        obj_dict = obj.__dict__
-        func_name = self._fget.func_name
         if self._fset:
-            ret = self._fset(obj, val)
-            if ret is not None and obj_dict.get(func_name) is ret:
-                # By returning the value set the setter signals that it took
-                # over setting the value in obj.__dict__; this mechanism allows
-                # it to override the input value
-                return
-        obj_dict[func_name] = val
+            self._fset(obj, val)
+        obj.__dict__[self._fget.func_name] = val
 
     def __delete__(self, obj):
         if self._fdel:
@@ -299,18 +114,42 @@ class lazyproperty(object):
         if key in obj.__dict__:
             del obj.__dict__[key]
 
+    def getter(self, fget):
+        return self.__ter(fget, 0)
 
-def deprecated(message='', name='', alternative='', pending=False):
+    def setter(self, fset):
+        return self.__ter(fset, 1)
+
+    def deleter(self, fdel):
+        return self.__ter(fdel, 2)
+
+    def __ter(self, f, arg):
+        args = [self._fget, self._fset, self._fdel, self.__doc__]
+        args[arg] = f
+        cls_ns = sys._getframe(1).f_locals
+        for k, v in cls_ns.iteritems():
+            if v is self:
+                property_name = k
+                break
+
+        cls_ns[property_name] = lazyproperty(*args)
+
+        return cls_ns[property_name]
+
+
+# TODO: Provide a class deprecation marker as well.
+def deprecated(since, message='', name='', alternative='', pending=False):
     """
     Used to mark a function as deprecated.
 
-    TODO: Provide a class deprecation marker as well.
-
     To mark an attribute as deprecated, replace that attribute with a
     depcrecated property.
 
     Parameters
     ------------
+    since : str
+        The release at which this API became deprecated.  This is required.
+
     message : str, optional
         Override the default deprecation message.  The format specifier
         %(func)s may be used for the name of the function, and %(alternative)s
@@ -337,7 +176,8 @@ def deprecated(message='', name='', alternative='', pending=False):
 
     """
 
-    def deprecate(func):
+    def deprecate(func, message=message, name=name, alternative=alternative,
+                  pending=pending):
         if isinstance(func, classmethod):
             try:
                 func = func.__func__
@@ -357,36 +197,51 @@ def deprecated(message='', name='', alternative='', pending=False):
         else:
             is_classmethod = False
 
+        if not name:
+            name = func.__name__
+
+        altmessage = ''
+        if not message or type(message) == type(deprecate):
+            if pending:
+                message = ('The %(func)s function will be deprecated in a '
+                           'future version.')
+            else:
+                message = ('The %(func)s function is deprecated and may '
+                           'be removed in a future version.')
+            if alternative:
+                altmessage = '\n        Use %s instead.' % alternative
+
+        message = ((message % {'func': name, 'alternative': alternative}) +
+                   altmessage)
+
         @functools.wraps(func)
         def deprecated_func(*args, **kwargs):
-            # _message and _name are necessary; otherwise assignments to name
-            # and message will cause the interpreter to treat them as local
-            # variables, instead of encapuslated variables
-            _message = message
-            _name = name
-            if not _name:
-                _name = func.__name__
-
-            if not _message or type(_message) == type(deprecate):
-                if pending:
-                    _message = 'The %(func)s function will be deprecated in' \
-                               'a future version.'
-                else:
-                    _message = 'The %(func)s function is deprecated and may ' \
-                               'be removed in a future version.'
-                if alternative:
-                    _message += '  Use %(alternative)s instead.'
-
             if pending:
                 category = PendingDeprecationWarning
             else:
                 category = DeprecationWarning
 
-            warnings.warn(
-                _message % {'func': _name, 'alternative': alternative},
-                category, stacklevel=2)
+            warnings.warn(message, category, stacklevel=2)
 
             return func(*args, **kwargs)
+
+        old_doc = deprecated_func.__doc__
+        if not old_doc:
+            old_doc = ''
+        old_doc = textwrap.dedent(old_doc).strip('\n')
+        altmessage = altmessage.strip()
+        if not altmessage:
+            altmessage = message.strip()
+        new_doc = (('\n.. deprecated:: %(since)s'
+                    '\n    %(message)s\n\n' %
+                    {'since': since, 'message': altmessage.strip()}) + old_doc)
+        if not old_doc:
+            # This is to prevent a spurious 'unexected unindent' warning from
+            # docutils when the original docstring was blank.
+            new_doc += r'\ '
+
+        deprecated_func.__doc__ = new_doc
+
         if is_classmethod:
             deprecated_func = classmethod(deprecated_func)
         return deprecated_func
@@ -456,6 +311,25 @@ def pairwise(iterable):
     return itertools.izip(a, b)
 
 
+def isiterable(obj):
+    """Returns true of the given object is iterable."""
+
+    # In Python2.6 and up this is simply a matter of checking isinstance
+    # collections.Iterable, but this unavailable in Python 2.5 and below
+    try:
+        from collections import Iterable
+        if isinstance(obj, Iterable):
+            return True
+    except ImportError:
+        pass
+
+    try:
+        iter(obj)
+        return True
+    except TypeError:
+        return False
+
+
 def encode_ascii(s):
     """
     In Python 2 this is a no-op.  Strings are left alone.  In Python 3 this
@@ -579,22 +453,6 @@ def fileobj_closed(f):
         return True
 
 
-def fileobj_is_binary(f):
-    """
-    Returns True if the give file or file-like object has a file open in binary
-    mode.  When in doubt, returns True by default.
-    """
-
-    # TODO: In Python 3 it might be more reliable to check if the fileobj is a
-    # text reader or a binary reader
-
-    mode = fileobj_mode(f)
-    if mode:
-        return 'b' in mode
-    else:
-        return True
-
-
 def fileobj_mode(f):
     """
     Returns the 'mode' string of a file-like object if such a thing exists.
@@ -621,9 +479,29 @@ def fileobj_mode(f):
                 mode = mode.replace('a', 'r')
             else:
                 mode = mode.replace('a', 'w')
+    else:
+        # If mode still turned out to be something other than a string, it's
+        # not the droid we were looking for, and we should just toss it
+        mode = None
     return mode
 
 
+def fileobj_is_binary(f):
+    """
+    Returns True if the give file or file-like object has a file open in binary
+    mode.  When in doubt, returns True by default.
+    """
+
+    # TODO: In Python 3 it might be more reliable to check if the fileobj is a
+    # text reader or a binary reader
+
+    mode = fileobj_mode(f)
+    if mode:
+        return 'b' in mode
+    else:
+        return True
+
+
 def translate(s, table, deletechars):
     """
     This is a version of string/unicode.translate() that can handle string or
@@ -642,8 +520,29 @@ def translate(s, table, deletechars):
 
 
 def indent(s, shift=1, width=4):
-    return '\n'.join(' ' * (width * shift) + l if l else ''
-                     for l in s.splitlines())
+    indented = '\n'.join(' ' * (width * shift) + l if l else ''
+                         for l in s.splitlines())
+    if s[-1] == '\n':
+        indented += '\n'
+
+    return indented
+
+
+def fill(text, width, *args, **kwargs):
+    """
+    Like :func:`textwrap.wrap` but preserves existing paragraphs which
+    :func:`textwrap.wrap` does not otherwise handle well.  Also handles section
+    headers.
+    """
+
+    paragraphs = text.split('\n\n')
+    def maybe_fill(t):
+        if all(len(l) < width for l in t.splitlines()):
+            return t
+        else:
+            return textwrap.fill(t, width, *args, **kwargs)
+
+    return '\n\n'.join(maybe_fill(p) for p in paragraphs)
 
 
 def _array_from_file(infile, dtype, count, sep):
@@ -664,31 +563,10 @@ def _array_to_file(arr, outfile):
     """Write a numpy array to a file or a file-like object."""
 
     if isfile(outfile):
-        def write(a, f):
-            a.tofile(f)
+        arr.tofile(outfile)
     else:
         # treat as file-like object with "write" method
-        def write(a, f):
-            _write_string(f, a.tostring())
-
-    # Implements a workaround for a bug deep in OSX's stdlib file writing
-    # functions; on 64-bit OSX it is not possible to correctly write a number
-    # of bytes greater than 2 ** 32 and divisble by 4096 (or possibly 8192--
-    # whatever the default blocksize for the filesystem is).
-    # This issue should have a workaround in Numpy too, but hasn't been
-    # implemented there yet: https://github.com/astropy/astropy/issues/839
-    osx_write_limit = (2 ** 32) - 1
-
-    if (sys.platform == 'darwin' and platform.architecture()[0] == '64bit' and
-            arr.nbytes >= osx_write_limit + 1 and arr.nbytes % 4096 == 0):
-        idx = 0
-        # chunksize is a count of elements in the array, not bytes
-        chunksize = osx_write_limit // arr.itemsize
-        while idx < arr.nbytes:
-            write(arr[idx:idx + chunksize], outfile)
-            idx += chunksize
-    else:
-        write(arr, outfile)
+        _write_string(outfile, arr.tostring())
 
 
 def _write_string(f, s):
@@ -708,36 +586,16 @@ def _write_string(f, s):
     f.write(s)
 
 
-def _chunk_array(arr, CHUNK_SIZE=2 ** 25):
-    """
-    Yields subviews of the given array.  The number of rows is
-    selected so it is as close to CHUNK_SIZE (bytes) as possible.
-    """
-
-    if len(arr) == 0:
-        return
-    if isinstance(arr, FITS_rec):
-        arr = np.asarray(arr)
-    row_size = arr[0].size
-    rows_per_chunk = max(min(CHUNK_SIZE // row_size, len(arr)), 1)
-    for idx in range(0, len(arr), rows_per_chunk):
-        yield arr[idx:idx + rows_per_chunk,...]
-
-
 def _convert_array(array, dtype):
     """
     Converts an array to a new dtype--if the itemsize of the new dtype is
-    the same as the old dtype and both types are not numeric, a view is
-    returned.  Otherwise a new array must be created.
+    the same as the old dtype, a view is returned.  Otherwise a new array must
+    be created.
     """
 
     if array.dtype == dtype:
         return array
-    elif (array.dtype.itemsize == dtype.itemsize and not
-            (np.issubdtype(array.dtype, np.number) and
-             np.issubdtype(dtype, np.number))):
-        # Includes a special case when both dtypes are at least numeric to
-        # account for ticket #218: https://trac.assembla.com/pyfits/ticket/218
+    elif array.dtype.itemsize == dtype.itemsize:
         return array.view(dtype)
     else:
         return array.astype(dtype)
@@ -824,6 +682,45 @@ def _normalize_slice(input, naxis):
     return slice(_start, _stop, _step)
 
 
+def _words_group(input, strlen):
+    """
+    Split a long string into parts where each part is no longer
+    than `strlen` and no word is cut into two pieces.  But if
+    there is one single word which is longer than `strlen`, then
+    it will be split in the middle of the word.
+    """
+
+    words = []
+    nblanks = input.count(' ')
+    nmax = max(nblanks, len(input) // strlen + 1)
+    arr = np.fromstring((input + ' '), dtype=(bytes, 1))
+
+    # locations of the blanks
+    blank_loc = np.nonzero(arr == ' '.encode('latin1'))[0]
+    offset = 0
+    xoffset = 0
+    for idx in range(nmax):
+        try:
+            loc = np.nonzero(blank_loc >= strlen + offset)[0][0]
+            offset = blank_loc[loc - 1] + 1
+            if loc == 0:
+                offset = -1
+        except:
+            offset = len(input)
+
+        # check for one word longer than strlen, break in the middle
+        if offset <= xoffset:
+            offset = xoffset + strlen
+
+        # collect the pieces in a list
+        words.append(input[xoffset:offset])
+        if len(input) == offset:
+            break
+        xoffset = offset
+
+    return words
+
+
 def _tmp_name(input):
     """
     Create a temporary file name which should not already exist.  Use the
@@ -837,29 +734,51 @@ def _tmp_name(input):
     return fn
 
 
-if sys.version_info[:2] < (2, 6):
-    # In Python 2.5 mmap.mmap is a function that returns an object of type
-    # 'mmap.mmap', but the mmap.mmap type is otherwise not accessible through
-    # the module
-    def _is_mmap(obj):
-        return (type(obj).__module__ == 'mmap' and
-                type(obj).__name__ == 'mmap')
-else:
-    def _is_mmap(obj):
-        return isinstance(obj, mmap.mmap)
-
-
-def _get_array_mmap(array):
+def _get_array_memmap(array):
     """
-    If the array has an mmap.mmap at base of its base chain, return the mmap
-    object; otherwise return None.
+    If the array has a numpy.memmap as one of its bases, return the memmap
+    base; otherwise return None.
     """
 
-    if _is_mmap(array):
+    if isinstance(array, Memmap):
         return array
 
     base = array
     while hasattr(base, 'base') and base.base is not None:
-        if _is_mmap(base.base):
+        if isinstance(base.base, Memmap):
             return base.base
         base = base.base
+
+
+if sys.version_info[:2] < (2, 6):
+    import __builtin__
+    # Replace the builtin property to add support for the getter/setter/deleter
+    # mechanism as introduced in Python 2.6 (this can go away if we ever drop
+    # 2.5 support)
+    class property(property):
+        def __init__(self, fget, *args, **kwargs):
+            self.__doc__ = fget.__doc__
+            super(property, self).__init__(fget, *args, **kwargs)
+
+        def getter(self, fget):
+            return self.__ter(fget, 0)
+
+        def setter(self, fset):
+            return self.__ter(fset, 1)
+
+        def deleter(self, fdel):
+            return self.__ter(fdel, 2)
+
+        def __ter(self, f, arg):
+            args = [self.fget, self.fset, self.fdel, self.__doc__]
+            args[arg] = f
+            cls_ns = sys._getframe(1).f_locals
+            for k, v in cls_ns.iteritems():
+                if v is self:
+                    property_name = k
+                    break
+
+            cls_ns[property_name] = property(*args)
+
+            return cls_ns[property_name]
+    __builtin__.property = property
